"""Test vedirect_app module."""
import os
import time
import pytest
from ve_utils.utype import UType as Ut
from vedirect_m8.exceptions import InputReadException
from vemonitor_m8.core.exceptions import SettingInvalidException
from vemonitor_m8.workers.vedirect.vedirect_app import VedirectApp


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = None

        def init_vedirect_app(self):
            """Init WorkersDict class"""
            self.obj = VedirectApp(
                serial_conf={
                    "serial_port": os.path.join(
                        os.path.expanduser('~'),
                        'vmodem1'
                    )
                },
                serial_test={
                    "PIDTest": {
                        "typeTest": "value",
                        "key": "PID",
                        "value": "0x203"
                    }
                },
                source_name="PyTest",
                auto_start=False,
                wait_connection=True,
                wait_timeout=5
            )

        def read_serial_data(self):
            """Read Serial Data"""
            assert self.obj.ve.try_serial_connection(  # type: ignore
                "PyTest"
            ) is True
            assert self.obj.ve.has_data_cache() is False  # type: ignore
            result, is_cache = self.obj.ve.read_data(  # type: ignore
                caller_name="PyTest",
                timeout=2
            )
            assert Ut.is_dict(result, eq=26)
            assert is_cache is False
            assert self.obj.ve.has_data_cache() is True  # type: ignore

    return HelperManager()


class TestVedirectApp:
    """Test VedirectApp class."""

    def test_lock_serial(self, helper_manager):
        """Test lock_serial method """
        helper_manager.init_vedirect_app()
        helper_manager.read_serial_data()
        # Test lock serial
        result = helper_manager.obj.lock_serial(
            caller_name="PyTest"
        )
        assert result is True

        # Test init lock serial
        result = helper_manager.obj.init_lock_serial(
            caller_name="PyTest"
        )
        assert result is True

        # Test is serial locked by actual caller
        result = helper_manager.obj.is_serial_locked_by_caller(
            caller_name="PyTest"
        )
        assert result is True
        # Test is serial locked by bad caller
        result = helper_manager.obj.is_serial_locked_by_caller(
            caller_name="BadCaller"
        )
        assert result is False

        # Test is serial locked
        result = helper_manager.obj.is_serial_locked()
        assert result is True

        # unlock serial
        helper_manager.obj.unlock_serial()

        # Test is serial locked by actual caller
        result = helper_manager.obj.is_serial_locked_by_caller(
            caller_name="PyTest"
        )
        assert result is False
        # Test is serial locked
        result = helper_manager.obj.is_serial_locked()
        assert result is False

        # Test init lock serial
        result = helper_manager.obj.init_lock_serial(
            caller_name="PyTest"
        )
        assert result is True

        # Test is serial locked
        result = helper_manager.obj.is_serial_locked()
        assert result is True

        # unlock serial
        helper_manager.obj.unlock_serial()

        # Test init lock serial with bad caller name
        with pytest.raises(SettingInvalidException):
            helper_manager.obj.init_lock_serial(
                caller_name=None
            )

    def test_read_data(self, helper_manager):
        """Test is_time_to_read_serial method """
        helper_manager.init_vedirect_app()
        helper_manager.read_serial_data()
        # Read data from cache
        # Interval is less than 1s
        result = helper_manager.obj.read_data(
            caller_name="PyTest",
            timeout=2
        )
        time.sleep(1)
        # Read data from serial
        result = helper_manager.obj.read_data(
            caller_name="PyTest",
            timeout=2
        )
        assert Ut.is_dict(result, eq=26)
        time.sleep(1)
        # try exception by max read errors
        assert helper_manager.obj.ve.packets_stats.set_max_read_error(
            value=2
        ) is True
        helper_manager.obj.ve.packets_stats.reset_global_counters()
        assert helper_manager.obj.ve.packets_stats.add_serial_read_errors(
        ) == 1
        assert helper_manager.obj.ve.packets_stats.add_serial_read_errors(
        ) == 2
        with pytest.raises(InputReadException):
            helper_manager.obj.read_data(
                caller_name="PyTest",
                timeout=2
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test Utils class."""
import time
from vemonitor_m8.core.utils import Utils as Ut


class TestUtils:
    """Test Utils class."""

    def test_get_min_in_loop(self):
        """Test check_columns method."""
        result = Ut.get_min_in_loop(
            value=0,
            min_val=6
        )
        assert result == 6

        result = Ut.get_min_in_loop(
            value=12,
            min_val=6
        )
        assert result == 6

        result = Ut.get_min_in_loop(
            value=3,
            min_val=6
        )
        assert result == 3

    def test_get_max_in_loop(self):
        """Test get_max_in_loop method."""
        result = Ut.get_max_in_loop(
            value=0,
            max_val=6
        )
        assert result == 6

        result = Ut.get_max_in_loop(
            value=3,
            max_val=6
        )
        assert result == 6

        result = Ut.get_max_in_loop(
            value=12,
            max_val=6
        )
        assert result == 12

    def test_all_equal(self):
        """Test all_equal method."""
        result = Ut.all_equal(
            iterable=[1, 1, 1, 1, 1]
        )
        assert result is True

        result = Ut.all_equal(
            iterable=[1, 1, 2, 1, 1]
        )
        assert result is False

        result = Ut.all_equal(
            iterable=(1, 1, 1, 1, 1)
        )
        assert result is True

        result = Ut.all_equal(
            iterable=(1, 1, 2, 1, 1)
        )
        assert result is False

        result = Ut.all_equal(
            iterable={'a': 1, 'b': 1, 'c': 1, 'd': 1}
        )
        assert result is False

    def test_is_valid_port(self):
        """Test is_valid_port method."""
        assert Ut.is_valid_port(0) is False
        assert Ut.is_valid_port(-1) is False
        assert Ut.is_valid_port(65536) is False
        assert Ut.is_valid_port(1) is True
        assert Ut.is_valid_port(65535) is True

    def test_is_valid_host(self):
        """Test is_valid_host method."""
        assert Ut.is_valid_host(0) is False
        assert Ut.is_valid_host('0') is False
        assert Ut.is_valid_host('0.0') is False
        assert Ut.is_valid_host('0.0.0') is False
        assert Ut.is_valid_host('256.255.255.255') is False
        assert Ut.is_valid_host('255.256.255.255') is False
        assert Ut.is_valid_host('255.255.256.255') is False
        assert Ut.is_valid_host('255.255.255.256') is False
        assert Ut.is_valid_host('-1.255.255.255') is False
        assert Ut.is_valid_host('255.-1.255.255') is False
        assert Ut.is_valid_host('255.255.-1.255') is False
        assert Ut.is_valid_host('255.255.255.-1') is False

        assert Ut.is_valid_host('0.0.0.0') is True
        assert Ut.is_valid_host('255.255.255.255') is True
        assert Ut.is_valid_host('127.0.0.1') is True
        assert Ut.is_valid_host('192.168.1.1') is True

    def test_is_from_time(self):
        """Test is_from_time method."""
        assert Ut.is_from_time(
            item_time=2,
            from_time=1
        ) is True

        assert Ut.is_from_time(
            item_time=2,
            from_time=0
        ) is True

        assert Ut.is_from_time(
            item_time=0,
            from_time=1
        ) is False

        assert Ut.is_from_time(
            item_time=-1,
            from_time=1
        ) is False

        assert Ut.is_from_time(
            item_time=1,
            from_time=-1
        ) is False

        assert Ut.is_from_time(
            item_time="a",
            from_time=1
        ) is False

        assert Ut.is_from_time(
            item_time=1,
            from_time="a"
        ) is False

        assert Ut.is_from_time(
            item_time=None,
            from_time=1
        ) is False

        assert Ut.is_from_time(
            item_time=1,
            from_time=None
        ) is False




"""Test ConfigLoader class."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.conf_manager.config_loader import ConfigLoader
from vemonitor_m8.core.exceptions import YAMLFileNotFound
from vemonitor_m8.models.config import Config


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                        inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )
            self.obj = ConfigLoader(self.test_path)

    return HelperManager()


class TestConfigLoader():
    """Test ConfigLoader class."""

    def test_set_settings_from_files(self, helper_manager):
        """Test set_settings_from_files method"""
        settings = helper_manager.obj.set_settings_from_files(
            app_name="batSerialMonitor"
        )
        assert isinstance(
            settings,
            Config
        )
        assert Config.is_app_block(
            settings.app_blocks[0]
        ) and settings.app_blocks[0].get('app') == "batSerialMonitor"
        assert settings.has_battery_banks()

    def test_get_yaml_data_structure(self, helper_manager):
        """Test get_app_blocks_by_app_or_name method """
        assert Ut.is_dict(helper_manager.obj.get_yaml_data_structure())

        with pytest.raises(YAMLFileNotFound):
            helper_manager.obj.get_yaml_data_structure("userDeviceData.yaml")

    def test_get_settings_from_schema(self, helper_manager):
        """Test get_settings_from_schema method"""
        settings = helper_manager.obj.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        assert isinstance(
            settings,
            Config
        ) and settings.is_valid() and len(settings.app_blocks) == 1
        assert Config.is_app_block(
            settings.app_blocks[0]
        ) and settings.app_blocks[0].get('app') == "batSerialMonitor"
        assert settings.has_battery_banks() and settings.has_data_structures()



"""Test ThreadsController method."""


import time
from ve_utils.utype import UType as Ut
from vemonitor_m8.core.threads_controller import ThreadsController


def timers_callback(**kwargs):
    """Timer callback function"""
    time_call = time.time()
    timer_list = kwargs.get('timer_list')
    timer_list.append(time_call)


class TestThreadsController:
    """Test ThreadsController method."""

    def test_run_timers(self):
        """Test validate_data method."""
        obj = ThreadsController()
        timer_list = []
        obj.add_timer_key(
            key="Timer1",
            interval=1,
            callback=timers_callback,
            kwargs={'timer_list': timer_list}
        )

        assert obj.has_timer_key("Timer1") is True

        obj.add_timer_key(
            key="Timer2",
            interval=2,
            callback=timers_callback,
            kwargs={'timer_list': timer_list}
        )
        assert obj.has_timer_key("Timer2") is True
        assert obj.can_add_threads() is True
        obj.start_timers()
        time.sleep(5)
        obj.cancel_all_timers()

        assert Ut.is_list(timer_list, not_null=True)
        assert round(timer_list[1] - timer_list[0], 0) == 1.0
        assert round(timer_list[2] - timer_list[1], 0) == 0.0
        assert round(timer_list[3] - timer_list[1], 0) == 1.0
        assert round(timer_list[4] - timer_list[3], 0) == 1.0
        assert round(timer_list[5] - timer_list[4], 0) == 0.0



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Used to load jsonschema and validate data with it.

Get data to test and jsonschema key,
Load the jsonschema file with the private class method _load_schema,
and validate the instance data.
If any validation error occurs, raise exception.
Or if no error occurs, return data tested object

:Example:
    > SchemaValidate.validate_data(
        data=appBlocks_data,
        shem_key_path="appBlocks"
    )
    > ...appBlocks_data...

 .. seealso:: ConfigLoader
 .. raises:: ValidationError, SchemaError, ErrorTree
"""
import inspect
import logging
from os import path as Opath
from jsonschema import validate
from jsonschema.exceptions import SchemaError

from ve_utils.ujson import UJson
from ve_utils.utype import UType as Ut

__author__ = "Eli Serra"
__copyright__ = "Copyright 2022, Eli Serra"
__deprecated__ = False
__license__ = "Apache"
__status__ = "Production"
__version__ = "0.0.1"

logging.basicConfig()
logger = logging.getLogger("vemonitor")


class SchemaValidate():
    """Simple Class to load jsonschema file and validate data."""

    MAX_FILE_SIZE = 80000

    @classmethod
    def validate_data(cls, data: any, shem_key_path: str) -> None:
        """
        Validate data with loaded jsonschema.

        It will import json schema file from shem_key_path,
        and if json schema is a dict, use jsonschema.validate.
        If any validation error occurs, raise exception.
        Or if no error occurs, return None

        :param cls:
            Used to Refer to the class itself,
            so that we can call its methods.
        :param data:any:
            Data to test with the json schema loaded file.
        :param shem_key_path:str:
            Key of the json schema file to load.
        :return: None if no error occurs.

        :Example:

            > SchemaValidate.validate_data(
                data=appBlocks_data,
                shem_key_path="appBlocks"
            )

        .. seealso::  ConfigLoader
        .. raises:: ValidationError, SchemaError, ErrorTree
        .. warnings:: Class Method and Public
        """
        schema = SchemaValidate.load_schema(shem_key_path)
        if Ut.is_dict(schema, not_null=True):
            return SchemaValidate.validate_data_from_schema(data, schema)
        raise SchemaError(
            "Fatal error: "
            f"unable to load valid jsonschema for key {shem_key_path}"
        )

    @classmethod
    def validate_data_from_schema(cls, data: any, schema: dict) -> None:
        """
        Validate data with provided jsonschema.

        if json schema is a dict, use jsonschema.validate.
        If any validation error occurs, raise exception.
        Or if no error occurs, return None

        :param cls:
            Used to Refer to the class itself,
            so that we can call its methods.
        :param data:any:
            Data to test with the json schema loaded file.
        :param schema:dict:
            the json schema to validate data.
        :return: None if no error occurs.

        :Example:

            > SchemaValidate.validate_data_from_schema(
                data=appBlocks_data,
                schema={....}
            )

        .. seealso::  ConfigLoader
        .. raises:: ValidationError, SchemaError, ErrorTree
        .. warnings:: Class Method and Public
        """
        if Ut.is_dict(schema, not_null=True):
            validate(instance=data, schema=schema)
            return data
        raise SchemaError(
            f"Fatal error: invalid jsonschema : {schema}"
        )

    @classmethod
    def load_schema(cls,
                    file_key: str
                    ):
        """
        Load json schema from file_key, and return parsed content.

        First get the file name, adding "_schema.json"
        at the end of file_key value.
        Then use the current script path to locate the schema directory path.
        And finaly, if path is a file on disck,
        read and parse the file content.
            > ...vemonitor/conf_manager/schemas/[file_key]_schema.json

        If any error occurs, raise exception.
        Or if no error occurs, return parsed json schema.

        :param cls:
            Used to Refer to the class itself,
            so that we can call its methods.
        :param file_key:str:
            Key of the json schema file to load.
        :return: parsed json schema data or None if any error occurs.

        :Example:

            > SchemaValidate._load_schema(shem_key_path="appBlocks")
            > {my_schema_data}

        .. seealso::  ConfigLoader
        .. raises:: ValidationError, SchemaError, ErrorTree
        .. warnings:: Class Method and Private
        """
        if not Ut.is_str(file_key, not_null=True) or Opath.isabs(file_key):
            raise SchemaError(
                f"Fatal error, jsonschema for key {file_key}."
                "is not valid"
            )

        current_script_path = Opath.dirname(
            Opath.abspath(inspect.getfile(inspect.currentframe()))
        )
        file_path = f"{file_key}_schema.json"
        path = Opath.join(current_script_path, 'schemas', file_path)
        if not Opath.isfile(path):
            raise SchemaError(
                f"Fatal error, unable to load jsonschema for key {file_key}."
                f"Path : {path}"
            )

        file_size = Opath.getsize(path)
        if file_size > cls.MAX_FILE_SIZE:
            raise SchemaError(
                "Fatal error jsonschema file is too big, "
                f"file size : {file_size}/{cls.MAX_FILE_SIZE}"
            )

        data = None
        # load the yaml file
        with open(path, "r", encoding="utf-8") as f:
            data = UJson.load_json(f)

        return data



"""Test __init__ module"""
import logging
from vemonitor_m8 import configure_logging

logging.basicConfig()
logger = logging.getLogger("vemonitor")


class TestInit:
    """Test __init__ module"""

    def test_logger_debug(self):
        """Test Logger debug"""
        configure_logging(True)
        assert logger.level == logging.DEBUG

    def test_logger_default(self):
        """Test Logger Default"""
        configure_logging()
        assert logger.level == logging.INFO



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Workers models helper.
This module contain worker models and helpers.
    - Worker(ABC) Worker base class
    - InputWorker(Worker) Input Worker base class
    - OutputWorker(Worker) Output Worker base class
    - WorkersDict Workers items container class
    - Workers controller class
    - WorkersHelper helper class
"""
import time
from abc import ABC, abstractmethod
from typing import Optional, Union
from ve_utils.utype import UType as Ut

from vemonitor_m8.models.item_dict import DictOfObject

__author__ = "Eli Serra"
__copyright__ = "Copyright 2022, Eli Serra"
__deprecated__ = False
__license__ = "Apache"
__status__ = "Production"
__version__ = "1.0.0"


class Worker(ABC):
    """Worker model helper"""

    def __init__(self):
        self._info = {
            'name': "",
            'worker_key': "",
            'enum_key': -1
        }
        self.time_interval = None
        self.columns = None
        self.ref_cols = None
        self.worker = None
        self._status = False

    def close(self) -> bool:
        """Properly close worker instance."""
        return True

    def get_worker_status(self) -> bool:
        """Test if instance has name property."""
        return self._status is True

    def has_info(self) -> bool:
        """Test if instance has _info properties."""
        return self.has_name()\
            and self.has_worker_key()\
            and self.has_enum_key()

    def has_name(self) -> bool:
        """Test if instance has name property."""
        return Ut.is_str(self._info.get('name'), not_null=True)

    def get_name(self) -> str:
        """Get name property."""
        return self._info.get('name')

    def set_name(self, value: str) -> bool:
        """Set name property."""
        result = False
        if Ut.is_str(value, not_null=True):
            self._info['name'] = value
            result = True
        return result

    def has_worker_key(self) -> bool:
        """Test if instance has worker_key property."""
        return Ut.is_str(self._info.get('worker_key'), not_null=True)

    def get_worker_key(self) -> str:
        """Get worker_key property."""
        return self._info.get('worker_key')

    def set_worker_key(self, value: str) -> bool:
        """Set worker_key property."""
        result = False
        if Ut.is_str(value, not_null=True):
            self._info['worker_key'] = value
            result = True
        return result

    def has_enum_key(self) -> bool:
        """Test if instance has enum_key property."""
        return Ut.is_int(self._info.get('enum_key'), mini=0)

    def get_enum_key(self) -> int:
        """Get enum_key property."""
        return self._info.get('enum_key')

    def set_enum_key(self, value: int) -> bool:
        """Set enum_key property."""
        result = False
        if Ut.is_int(value, mini=0):
            self._info['enum_key'] = value
            result = True
        return result

    def has_time_interval(self) -> bool:
        """Test if instance has time_interval property."""
        return Ut.is_numeric(self.time_interval, mini=0)

    def set_time_interval(self, value: Union[int, float]) -> bool:
        """Set time_interval property."""
        result = False
        if Ut.is_numeric(value, positive=True):
            self.time_interval = value
            result = True
        return result

    def has_ref_cols(self) -> bool:
        """Test if instance has ref_cols property."""
        return Ut.is_list(self.ref_cols, not_null=True)

    def set_ref_cols(self, value: list) -> bool:
        """Set ref_cols property."""
        result = False
        if Ut.is_list(value, not_null=True):
            self.ref_cols = value
            result = True
        return result

    def has_worker_conf(self) -> bool:
        """Test if instance has worker_conf properties."""
        return self.has_info() and self.has_time_interval()

    def set_worker_conf(self,
                        conf: Optional[dict]
                        ) -> bool:
        """
        Set Worker configuration data.

        the conf dictionary must contain :
            - name: str: required
            - worker_key: str: required
            - enum_key: int: required
            - time_interval: Union[int, float]: required
            - columns: any: optional (see set_columns extended method)
            - ref_cols: list: optional

        """
        result = False
        if Ut.is_dict(conf)\
                and self.set_name(conf.get('name'))\
                and self.set_worker_key(conf.get('worker_key'))\
                and self.set_enum_key(conf.get('enum_key'))\
                and self.set_time_interval(conf.get('time_interval')):
            self.set_columns(conf.get('columns'))
            self.set_ref_cols(conf.get('ref_cols'))
            result = True
        return result

    @abstractmethod
    def is_ready(self) -> bool:
        """Test if Worker is ready."""

    @abstractmethod
    def set_worker_status(self) -> bool:
        """Test if Worker status is ready."""

    @abstractmethod
    def set_worker(self, worker: dict) -> bool:
        """Set Worker class instance."""

    @abstractmethod
    def has_columns(self) -> bool:
        """Test if instance has columns property."""

    @abstractmethod
    def set_columns(self, value) -> bool:
        """Set columns property."""


class InputWorker(Worker):
    """
    Input Worker model helper.
    Used by simple inputs workers
    """

    def __init__(self):
        Worker.__init__(self)

    def has_columns(self) -> bool:
        """Test if instance has columns property."""
        return Ut.is_list(self.columns, not_null=True)

    def set_columns(self, value: list) -> bool:
        """Set time_interval property."""
        result = False
        if Ut.is_list(value, not_null=True):
            self.columns = value
            result = True
        return result

    @abstractmethod
    def read_data(self,
                  timeout: int = 2
                  ) -> dict:
        """Get input data from Worker class instance."""


class InputDictWorker(InputWorker):
    """
    Input Worker model helper.
    Used by simple inputs workers
    """

    def __init__(self):
        InputWorker.__init__(self)

    def has_columns(self) -> bool:
        """Test if instance has columns property."""
        return Ut.is_dict(self.columns, not_null=True)

    def set_columns(self, value: dict) -> bool:
        """Set time_interval property."""
        result = False
        if Ut.is_dict(value, not_null=True):
            self.columns = value
            result = True
        return result


class OutputWorker(Worker):
    """Output Worker model helper"""

    def __init__(self):
        Worker.__init__(self)
        self.last_saved_time = 0
        self.cache_interval = 0
        self.min_req_interval = 0
        self.last_req = 0

    def has_columns(self) -> bool:
        """Test if instance has columns property."""
        return Ut.is_dict(self.columns, not_null=True)

    def set_columns(self, value: dict) -> bool:
        """Set time_interval property."""
        result = False
        if Ut.is_dict(value, not_null=True):
            self.columns = value
            result = True
        return result

    def has_last_saved_time(self) -> bool:
        """Test if instance has last_saved_time property."""
        return Ut.is_int(self.last_saved_time, positive=True)

    def get_last_saved_time(self) -> int:
        """Get last_saved_time property."""
        return Ut.get_int(self.last_saved_time, 0)

    def set_last_saved_time(self, value: Union[int, float]) -> bool:
        """Set last_saved_time property."""
        value = Ut.get_int(value, -1)
        result = False
        if Ut.is_int(value, mini=0):
            self.last_saved_time = value
            result = True
        return result

    def has_cache_interval(self) -> bool:
        """Test if instance has cache_interval property."""
        return Ut.is_int(self.cache_interval, positive=True)

    def get_cache_interval(self) -> int:
        """Test if instance has cache_interval property."""
        return Ut.get_int(self.cache_interval, 0)

    def set_cache_interval(self, value: int) -> bool:
        """Test if instance has cache_interval property."""
        result = False
        if Ut.is_int(value, mini=0):
            self.cache_interval = value
            result = True
        return result

    def has_min_req_interval(self) -> bool:
        """Test if instance has min_req_interval property."""
        return Ut.is_numeric(self.min_req_interval, positive=True)

    def get_min_req_interval(self) -> float:
        """Test if instance has min_req_interval property."""
        return Ut.get_float(self.min_req_interval, 0)

    def set_min_req_interval(self, value: Union[int, float]) -> bool:
        """Test if instance has min_req_interval property."""
        result = False
        if Ut.is_numeric(value, positive=True):
            self.min_req_interval = value
            result = True
        return result

    def update_req_time(self):
        """Update request time."""
        self.last_req = time.time()

    def respect_req_interval(self):
        """Update request time."""
        if self.has_min_req_interval()\
                and Ut.is_numeric(self.last_req, positive=True):
            interval = time.time() - self.last_req
            if interval < self.min_req_interval:
                time.sleep(self.min_req_interval - interval)

    def set_worker_conf(self,
                        conf: dict
                        ) -> bool:
        """
        Set Worker configuration data.

        the conf dictionary must contain :
            - name: str: required
            - worker_key: str: required
            - enum_key: int: required
            - time_interval: Union[int, float]: required
            - columns: dict: required
            - cache_interval: Union[int, float]: optional
            - ref_cols: list: optional

        """
        result = False
        if Ut.is_dict(conf)\
                and self.set_worker_key(conf.get('worker_key'))\
                and self.set_enum_key(conf.get('enum_key'))\
                and self.set_time_interval(conf.get('time_interval'))\
                and self.set_columns(conf.get('columns')):
            self.set_name(conf.get('name'))
            self.set_cache_interval(conf.get('cache_interval'))
            self.set_ref_cols(conf.get('ref_cols'))
            result = True
        return result

    @abstractmethod
    def send_data(self,
                  data: dict,
                  input_structure: dict
                  ) -> bool:
        """Send data to worker."""


class WorkersDict(DictOfObject):
    """Workers model helper"""
    def __init__(self):
        DictOfObject.__init__(self)

    def has_workers(self) -> bool:
        """Test if workers items defined."""
        return self.has_items()

    def has_worker_key(self, key: str) -> bool:
        """Test if instance has worker key defined."""
        return self.has_item_key(key=key)

    def init_worker(self, reset: bool = False):
        """Initialise worker items."""
        self.init_item(reset=reset)

    def add_worker(self,
                   key: str,
                   worker: Worker
                   ) -> bool:
        """Add worker item."""
        return self.add_item(
            key=key,
            value=worker
        )

    def get_worker(self, key: str) -> Optional[Worker]:
        """Get worker key."""
        return self.get_item(key=key)

    def loop_on_workers(self):
        """Get worker key."""
        if self.has_workers():
            for key, worker in self.items.items():
                if WorkersHelper.is_worker(worker):
                    yield key, worker

    def get_workers(self) -> Optional[dict]:
        """Get worker key."""
        return self.get_items()

    @staticmethod
    def is_valid_item_key(key: tuple):
        """Test if is valid item type"""
        return Ut.is_str(key)

    @staticmethod
    def is_valid_item_value(value: tuple):
        """Test if is valid item type"""
        return WorkersHelper.is_worker(value)


class Workers:
    """Workers model helper"""
    def __init__(self):
        self.inputs = WorkersDict()
        self.outputs = WorkersDict()

    def has_input_workers(self) -> bool:
        """Test if instance has worker items defined."""
        return WorkersHelper.is_workers_items(self.inputs)\
            and self.inputs.has_workers()

    def has_input_worker_key(self, key: str) -> bool:
        """Test if instance has input worker key defined."""
        return WorkersHelper.is_workers_items(self.inputs)\
            and self.inputs.has_worker_key(key)

    def add_input_worker(self,
                         key: str,
                         worker: InputWorker
                         ) -> bool:
        """Add input worker item."""
        result = False
        if WorkersHelper.is_workers_items(self.inputs)\
                and WorkersHelper.is_input_worker(worker)\
                and self.inputs.add_worker(key=key, worker=worker):
            result = True
        return result

    def get_input_worker(self, key: str) -> Optional[InputWorker]:
        """Get input worker by key."""
        result = None
        if WorkersHelper.is_workers_items(self.inputs):
            result = self.inputs.get_worker(key)
        return result

    def get_input_workers(self) -> Optional[dict]:
        """Get input workers."""
        result = None
        if self.has_output_workers():
            result = self.inputs.get_workers()
        return result

    def loop_on_input_workers(self):
        """Get input workers."""
        if self.has_input_workers():
            for key, worker in self.inputs.loop_on_workers():
                if WorkersHelper.is_input_worker(worker):
                    yield key, worker

    def has_output_workers(self) -> bool:
        """Test if instance has output worker items defined."""
        return WorkersHelper.is_workers_items(self.outputs)\
            and self.outputs.has_workers()

    def has_output_worker_key(self, key: str) -> bool:
        """Test if instance has output worker key defined."""
        return WorkersHelper.is_workers_items(self.outputs)\
            and self.outputs.has_worker_key(key)

    def add_output_worker(self,
                          key: str,
                          worker: OutputWorker
                          ) -> bool:
        """Add output worker item."""
        result = False
        if WorkersHelper.is_workers_items(self.outputs)\
                and WorkersHelper.is_output_worker(worker)\
                and self.outputs.add_worker(key=key, worker=worker):
            result = True
        return result

    def get_output_worker(self, key: str) -> Optional[OutputWorker]:
        """Get output worker by key."""
        result = None
        if WorkersHelper.is_workers_items(self.outputs):
            result = self.outputs.get_worker(key)
        return result

    def get_output_workers(self) -> Optional[dict]:
        """Get output workers."""
        result = None
        if self.has_output_workers():
            result = self.outputs.get_workers()
        return result

    def loop_on_output_workers(self):
        """Get output workers."""
        if self.has_output_workers():
            for key, worker in self.outputs.loop_on_workers():
                if WorkersHelper.is_output_worker(worker):
                    yield key, worker


class WorkersHelper:
    """WorkersHelper model helper"""

    @staticmethod
    def is_worker(worker: Worker) -> bool:
        """Test if is a worker instance."""
        return isinstance(worker, Worker)

    @staticmethod
    def format_worker_conf(connector: Union[dict, object],
                           worker_key: str,
                           enum_key: int,
                           item: dict,
                           ) -> Optional[dict]:
        """Get formatted worker configuration data."""
        result = None
        if Ut.is_str(worker_key) \
                and Ut.is_int(enum_key) \
                and connector is not None \
                and Ut.is_dict(item, not_null=True):
            result = {
                'connector': connector,
                'worker_key': worker_key,
                'enum_key': enum_key,
                'item': item
            }
        return result

    @staticmethod
    def get_worker_conf_from_dict(conf: dict) -> Optional[dict]:
        """Test if is configuration data."""
        result = None
        if Ut.is_dict(conf, not_null=True) \
                and Ut.is_dict(conf.get('item'), not_null=True):
            result = {
                "name": conf['item'].get('name'),
                "worker_key": conf.get('worker_key'),
                "enum_key": conf.get('enum_key'),
                "time_interval": conf['item'].get('time_interval'),
                "cache_interval": conf['item'].get('cache_interval'),
                "columns": conf['item'].get('columns'),
                "ref_cols": conf['item'].get('ref_cols')
            }
        return result

    @staticmethod
    def is_worker_ready(worker: Worker) -> bool:
        """Test if the controller is ready"""
        return WorkersHelper.is_worker(worker) and worker.is_ready()

    @staticmethod
    def is_input_worker(worker: InputWorker) -> bool:
        """Test if is a worker instance."""
        return isinstance(worker, InputWorker)

    @staticmethod
    def is_output_worker(worker: OutputWorker) -> bool:
        """Test if is a worker instance."""
        return isinstance(worker, OutputWorker)

    @staticmethod
    def is_workers_items(items) -> bool:
        """Test if workers items defined."""
        return isinstance(items, WorkersDict)

    @staticmethod
    def get_worker_name(key: str, item: dict) -> str:
        """Get worker formatted key."""
        return f"{key}_{item.get('source')}_{item.get('name')}"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Init vemonitor_m8 App"""
import logging
import sys
import argparse
from vemonitor_m8.app_run import AppRun
from vemonitor_m8.version import VERSION

__author__ = "Eli Serra"
__copyright__ = "Copyright 2022, Eli Serra"
__deprecated__ = False
__license__ = "Apache"
__status__ = "Production"
__version__ = VERSION

logging.basicConfig()
logger = logging.getLogger("vemonitor")


def parse_args(args):
    """
    Parsing function
    :param args: arguments passed from the command line
    :return: return parser
    """
    # create arguments
    parser = argparse.ArgumentParser(
        description='Monitor From VE.Direct protocol'
    )
    parser.add_argument(
        '--block',
        help='Main block name to run in your configuration file.'
    )
    parser.add_argument(
        '--app',
        help='Main app to run'
    )
    parser.add_argument(
        '--conf_path', help='Main Configuration file path'
    )
    parser.add_argument(
        '--log_path', help='Console log file path'
    )
    parser.add_argument(
        "--debug", action='store_true',
        help="Show debug output"
    )
    parser.add_argument(
        '-v', '--version',
        action='version',
        version='VeMonitor ' + __version__
    )

    # parse arguments from script parameters
    return parser.parse_args(args)


def main():
    """Entry point of VeMonitor program."""
    # parse argument. the script name is removed
    try:
        parser = parse_args(sys.argv[1:])
    except SystemExit:
        sys.exit(1)

    # check if we want debug
    configure_logging(debug=parser.debug)
    logger.warning("---------- VeMonitor_m8 ----------")
    logger.debug("Args: %s", parser)

    AppRun(
        block=parser.block,
        app=parser.app
    )


class AppFilter(logging.Filter):
    """
    Class used to add a custom entry into the logger
    """

    def filter(self, record):
        record.app_version = f"vemonitor-{__version__}"
        return True


class CustomFormatter(logging.Formatter):
    """Logging custom formatter."""

    grey = '\x1b[38;21m'
    blue = '\x1b[38;5;39m'
    yellow = '\x1b[38;5;226m'
    red = '\x1b[38;5;196m'
    bold_red = '\x1b[31;1m'
    reset = '\x1b[0m'

    def __init__(self,
                 fmt: str,
                 date_format: str = "%Y-%m-%d %H:%M:%S"
                 ):
        super().__init__()
        self.fmt = fmt
        self.date_format = date_format
        self.colors_format = {
            logging.DEBUG: self.grey + self.fmt + self.reset,
            logging.INFO: self.blue + self.fmt + self.reset,
            logging.WARNING: self.yellow + self.fmt + self.reset,
            logging.WARN: self.yellow + self.fmt + self.reset,
            logging.ERROR: self.red + self.fmt + self.reset,
            logging.CRITICAL: self.bold_red + self.fmt + self.reset
        }

    def format(self, record):
        log_fmt = self.colors_format.get(record.levelno)
        formatter = logging.Formatter(log_fmt, self.date_format)
        return formatter.format(record)


def configure_logging(debug=None):
    """
    Prepare log folder in current home directory.

    :param debug: If true, set the lof level to debug

    """
    log = logging.getLogger("vemonitor")
    log.addFilter(AppFilter())
    log.propagate = False
    syslog = logging.StreamHandler()
    syslog.setLevel(logging.DEBUG)

    formatter = CustomFormatter(
        '%(asctime)s :: %(app_version)s :: %(message)s', "%Y-%m-%d %H:%M:%S"
    )
    syslog.setFormatter(formatter)

    if debug:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)

    # add the handlers to logger
    log.addHandler(syslog)

    log.debug("Logger ready")



# -*- coding: utf-8 -*-
"""Vedirectsim example."""
import logging
import argparse
import sys

from vedirect_m8.vedirectsim import Vedirectsim
from vedirect_m8 import configure_logging

logging.basicConfig()
logger = logging.getLogger("vedirect")


def parse_args(args):
    """
    Parsing arguments.

    :param args: arguments passed from the command line
    :return: return parser
    """
    parser_arguments = argparse.ArgumentParser(
        description='A simple VE.Direct simulator')
    parser_arguments.add_argument('--port', help='Serial port')
    parser_arguments.add_argument('--device', help='Serial port')
    parser_arguments.add_argument(
        '--debug', action='store_true', help='Show debug output')

    # parse arguments from script parameters
    return parser_arguments.parse_args(args)


if __name__ == '__main__':

    parser_args = parse_args(sys.argv[1:])

    configure_logging(parser_args.debug)

    ve = Vedirectsim(parser_args.port, parser_args.device)
    ve.run()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Used to load yaml configuration from files.

 It was inspired from part of kaliope project.
     - Project url:
         https://github.com/kalliope-project/kalliope
     - Licence:
         https://github.com/kalliope-project/kalliope/blob/master/LICENSE.md

 .. seealso:: Loader
 .. raises:: YAMLFileNotFound, YAMLFileEmpty, YAMLFileError
"""
import os
import inspect
import logging
from typing import Optional, Union
from ve_utils.utype import UType as Ut
from ve_utils.usys import USys as Usys
from vemonitor_m8.conf_manager.yaml_loader import YmlConfLoader
from vemonitor_m8.core.exceptions import YAMLFileNotFound

__author__ = "Eli Serra"
__copyright__ = "Copyright 2020, Eli Serra"
__deprecated__ = False
__license__ = "Apache"
__status__ = "Production"
__version__ = "0.0.1"

logging.basicConfig()
logger = logging.getLogger("vemonitor")


class Loader():
    """
    This Class is used to be
    """
    def __init__(self,
                 file_names: Union[str, list, tuple],
                 file_path: Optional[str] = None
                 ):
        self.settings = None
        self.file_path = None
        self.set_file_path(
            file_name=file_names,
            base_path=file_path
        )

    def set_file_path(self,
                      file_name: Union[str, list, tuple],
                      base_path: Optional[str] = None
                      ):
        """
            Used to set the file path of a given file.
            The function takes one argument, which is either
            a string or list/tuple of strings.
            If the argument is a string, it will be interpreted
            as an absolute or relative path to a file that exists
            on disk. If the argument is a tuple/list,
            each item in that list must be either another tuple/list
            containing multiple strings
            (e.g., ['file_name', 'sub_dir', 'etc']),
            or just one string (e.g., ['file_name']).
            The first item in this nested structure must be
            the name of an existing file somewhere on disk;
            if there are more than one items in this nested
            structure then any number of them can be passed
            and they will all attempt to be joined together
            into one full path by os-specific rules for joining paths together;
            if there are no items then only os-specific rules
            for joining paths together will apply and nothing else.

            :param self: Used to Access variables that belongs to the class.
            :param file_name: Used to Set the file path.
            :return: The file path of the file name that is passed in.

            :doc-author: Trelent
        """
        self.file_path = None
        if not Ut.is_str(base_path, not_null=True)\
                or not os.path.isdir(base_path):
            base_path = None

        if Ut.is_str(file_name):
            if Ut.is_str(base_path):
                gpath = os.path.join(base_path, file_name)
                if os.path.isfile(gpath):
                    self.file_path = gpath

            if self.file_path is None:
                self.file_path = Loader.get_real_file_path(file_name)

        elif Ut.is_tuple(file_name) or Ut.is_list(file_name):
            tmp = None
            for name in file_name:
                if Ut.is_str(base_path):
                    tmp = os.path.join(base_path, name)
                    if os.path.isfile(tmp):
                        self.file_path = tmp
                        break

                if tmp is None:
                    tmp = Loader.get_real_file_path(name)

                    if Ut.is_str(tmp) and os.path.isfile(tmp):
                        self.file_path = tmp
                        break

        # if the returned file path is none, the file doesn't exist
        if self.file_path is None:
            raise YAMLFileNotFound(
                "[Loader::set_file_path] Fatal Error: "
                "Unable to set configuration file path "
                f"for file_name {file_name}"
                )

    def get_yaml_config(self,
                        child_list: Optional[list] = None
                        ) -> Optional[Union[dict, list]]:
        """
        Loads the provided YAML file from self.file_path
        and return it as a dict or list.

        In the main configuration file, child import can be done.
        Result is list or dict of configuration files content
        and batteryBank.yaml content if exist on same path.

        :Example :
            >>> config_yaml = self.get_yaml_config(
                child_list=['batteryBank.yaml']
            )
            >>> {...} # Content of yaml files (result can be a list)
        :param child_list: list or None: List of yaml file names to import,
            others are not.
        :return: list dict or None: The loaded YAML data
        """
        return YmlConfLoader.get_config(self.file_path, child_list)

    @staticmethod
    def get_real_file_path(file_path: Optional[str]) -> Optional[str]:
        """
            Try to return a full path from a given <file_path>
            If the path is an absolute on, we return it directly.

            If the path is relative, we try to get the full path in this order:
            - from the current directory where vemonitor has been called
                + the file_path.
            Eg: /home/me/Documents/vemonitor_config
            - from /etc/vemonitor + file_path
            - from the default file passed as <file_name>
                at the root of the project

            :param file_path file path to test
            :type file_path: str
            :return: absolute path to the file file_path
                or None if is doen't exist
        """
        result = None
        # ToDo: data must be sanitized
        if not os.path.isabs(file_path):

            paths_order = Loader.get_paths_order()

            for current_path in paths_order:
                sel_path = os.path.join(current_path, file_path)
                if os.path.isfile(sel_path):
                    logger.debug("File found in %s", current_path)
                    result = sel_path
                    break

        else:
            if os.path.isfile(file_path):
                result = file_path
        return result

    @staticmethod
    def get_current_script_path() -> str:
        """Get Current script path"""
        return os.path.dirname(
            os.path.abspath(
                inspect.getfile(inspect.currentframe())
            )
        )

    @staticmethod
    def get_paths_order() -> list:
        """Get Paths order to load config."""
        return [
            # Only linux
            os.path.join(os.path.sep, "opt", "vemonitor_m8", "conf"),
            os.path.join(os.path.sep, "opt", "vemonitor", "conf"),
            # Linux  and/or windows
            Usys.get_current_file_parent_parent_path(
                Loader.get_current_script_path()
            ),
            os.path.join(
                os.path.abspath(
                    os.path.expanduser("~")
                ),
                ".vemonitor"
            ),
            os.path.abspath(os.getcwd()),
        ]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test DataChecker class."""
import pytest
from vemonitor_m8.core.data_cache import DataCache


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            self.obj = DataCache(
                max_rows=10
            )

        def init_nodes_test(self):
            """Init DataCache nodes"""
            # reset data cache
            self.obj.reset_data_cache()
            # get all nodes list
            data = self.obj.get_cache_nodes_keys_list()
            assert data == []
            # register nodes
            assert self.obj.register_node('pytest_1') is True
            assert self.obj.register_node('pytest_2') is True
            assert self.obj.register_node('pytest_3') is True

        def add_data_test(self):
            """Init data to test"""
            now = 1722013440

            for i in range(50):
                now += 1
                assert self.obj.add_data_cache(
                    time_key=now,
                    node="pytest_1",
                    data={
                        'V': 26.8 + i + 1,
                        'I': 1.52 + i + 1
                    }
                ) is True

                if now % 2 == 0:
                    assert self.obj.add_data_cache(
                        time_key=now,
                        node="pytest_2",
                        data={
                            'V': 26.8 + i + 1,
                            'I': 1.52 + i + 1
                        }
                    ) is True

                if now % 5 == 0:
                    assert self.obj.add_data_cache(
                        time_key=now,
                        node="pytest_3",
                        data={
                            'V': 26.8 + i + 1,
                            'I': 1.52 + i + 1
                        }
                    ) is True

    return HelperManager()


class TestDataChecker:
    """Test DataChecker class."""

    def test_add_data_cache(self, helper_manager):
        """Test add_data_cache method."""
        # init nodes test
        helper_manager.init_nodes_test()
        # Set max data cache to 10 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_data_test()

        pytest_1_keys = helper_manager.obj.get_cache_keys_by_node(
            node='pytest_1'
        )
        pytest_2_keys = helper_manager.obj.get_cache_keys_by_node(
            node='pytest_2'
        )
        pytest_3_keys = helper_manager.obj.get_cache_keys_by_node(
            node='pytest_3'
        )
        assert len(pytest_1_keys) == 10
        assert len(pytest_2_keys) == 10
        assert len(pytest_3_keys) == 10

    def test_get_data_from_cache(self, helper_manager):
        """Test get_data_from_cache method."""
        # init nodes test
        helper_manager.init_nodes_test()
        # Set max data cache to 10 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_data_test()

        # get all data from cache
        result, last_time, max_time = helper_manager.obj.get_data_from_cache()
        assert len(result) == 22
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get fist for time  keys from data cache
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           nb_items=4
        )
        assert result == {
            1722013445: {
                'pytest_3': {'V': 31.8, 'I': 6.52}
            },
            1722013450: {
                'pytest_3': {'V': 36.8, 'I': 11.52}
            },
            1722013455: {
                'pytest_3': {'V': 41.8, 'I': 16.52}
            },
            1722013460: {
                'pytest_3': {'V': 46.8, 'I': 21.52}
            }
        }
        assert max_time == 1722013460
        assert last_time == 1722013461

        # get all items from time 1722013488
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           from_time=1722013488
        )
        assert result == {
            1722013488: {
                'pytest_1': {'V': 74.8, 'I': 49.52},
                'pytest_2': {'V': 74.8, 'I': 49.52}
            },
            1722013489: {'pytest_1': {'V': 75.8, 'I': 50.52}},
            1722013490: {
                'pytest_1': {'V': 76.8, 'I': 51.52},
                'pytest_2': {'V': 76.8, 'I': 51.52},
                'pytest_3': {'V': 76.8, 'I': 51.52}
            }
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get all pytest_1 node data
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           structure={'pytest_1': ['V', 'I']}
        )
        assert result == {
            1722013481: {'pytest_1': {'V': 67.8, 'I': 42.52}},
            1722013482: {'pytest_1': {'V': 68.8, 'I': 43.52}},
            1722013483: {'pytest_1': {'V': 69.8, 'I': 44.52}},
            1722013484: {'pytest_1': {'V': 70.8, 'I': 45.52}},
            1722013485: {'pytest_1': {'V': 71.8, 'I': 46.52}},
            1722013486: {'pytest_1': {'V': 72.8, 'I': 47.52}},
            1722013487: {'pytest_1': {'V': 73.8, 'I': 48.52}},
            1722013488: {'pytest_1': {'V': 74.8, 'I': 49.52}},
            1722013489: {'pytest_1': {'V': 75.8, 'I': 50.52}},
            1722013490: {'pytest_1': {'V': 76.8, 'I': 51.52}}
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get all 'V' values from pytest_1 node data
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           structure={'pytest_1': ['V']}
        )
        assert result == {
            1722013481: {'pytest_1': {'V': 67.8}},
            1722013482: {'pytest_1': {'V': 68.8}},
            1722013483: {'pytest_1': {'V': 69.8}},
            1722013484: {'pytest_1': {'V': 70.8}},
            1722013485: {'pytest_1': {'V': 71.8}},
            1722013486: {'pytest_1': {'V': 72.8}},
            1722013487: {'pytest_1': {'V': 73.8}},
            1722013488: {'pytest_1': {'V': 74.8}},
            1722013489: {'pytest_1': {'V': 75.8}},
            1722013490: {'pytest_1': {'V': 76.8}}
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get first four time keys data from time 1722013484
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           from_time=1722013484,
           nb_items=4
        )
        assert result == {
            1722013484: {
                'pytest_1': {'V': 70.8, 'I': 45.52},
                'pytest_2': {'V': 70.8, 'I': 45.52}
            },
            1722013485: {
                'pytest_1': {'V': 71.8, 'I': 46.52},
                'pytest_3': {'V': 71.8, 'I': 46.52}
            },
            1722013486: {
                'pytest_1': {'V': 72.8, 'I': 47.52},
                'pytest_2': {'V': 72.8, 'I': 47.52}
            },
            1722013487: {'pytest_1': {'V': 73.8, 'I': 48.52}}
        }
        assert max_time == 1722013487
        assert last_time == 1722013488

        # get first four pytest_1 nodes data from time 1722013484
        result, last_time, max_time = helper_manager.obj.get_data_from_cache(
           from_time=1722013484,
           nb_items=4,
           structure={'pytest_1': ['V', 'I']}
        )
        assert result == {
            1722013484: {'pytest_1': {'V': 70.8, 'I': 45.52}},
            1722013485: {'pytest_1': {'V': 71.8, 'I': 46.52}},
            1722013486: {'pytest_1': {'V': 72.8, 'I': 47.52}},
            1722013487: {'pytest_1': {'V': 73.8, 'I': 48.52}}
        }
        assert max_time == 1722013487
        assert last_time == 1722013488



"""
YmlConfLoader Unit Tests class
"""
import os
import inspect
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.conf_manager.yaml_loader import YmlConfLoader
from vemonitor_m8.core.exceptions import YAMLFileEmpty
from vemonitor_m8.core.exceptions import YAMLFileNotFound
from vemonitor_m8.core.exceptions import YAMLFileError


class TestYmlConfLoader():
    """YmlConfLoader Unit Tests class"""

    @staticmethod
    def get_dummy_conf_path():
        """Get dummy configuration files path for tests."""
        current_script_path = os.path.dirname(
            os.path.abspath(
                inspect.getfile(inspect.currentframe())
            )
        )
        return os.path.join(
            current_script_path,
            "conf"
        )

    def test_file_not_found(self):
        """Test get_config method YAMLFileNotFound Error."""
        # test main file not found
        with pytest.raises(YAMLFileNotFound):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "not_found.yaml"
                )
            )

        # test import file not found
        with pytest.raises(YAMLFileNotFound):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "import_not_found.yaml"
                )
            )

        # test import file exist but empty
        with pytest.raises(YAMLFileNotFound):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "import_empty.yaml"
                )
            )

    def test_file_error(self):
        """Test get_config method YAMLFileError Error."""
        # test import yaml file with list of data
        # when the main file config has dict of data
        with pytest.raises(YAMLFileError):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "dummy_conf_dict_error.yaml"
                )
            )

        # test import file with bad extension
        with pytest.raises(YAMLFileError):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "import_bad_ext.yaml"
                )
            )

        # Test get existing config file with bad extension
        with pytest.raises(YAMLFileError):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "bad_ext.ymld"
                )
            )

        # Test get existing but empty config file
        with pytest.raises(YAMLFileEmpty):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "empty.yaml"
                )
            )

    def test_get_dict_main_config(self):
        """Test get_config method with dict content ."""
        # test importing main dict configuration
        # with some child configuration
        conf = YmlConfLoader.get_config(
            os.path.join(
                TestYmlConfLoader.get_dummy_conf_path(),
                "dummy_conf_dict.yaml"
                )
        )
        assert Ut.is_dict(conf) and len(conf) == 7
        assert len(conf.get('Imports')) == 2
        assert len(conf.get('Dummy_1')) == 2
        assert len(conf.get('Dummy_2')) == 2
        assert len(conf.get('Dummy_3')) == 2
        assert len(conf.get('Dummy_4')) == 2
        assert len(conf.get('Dummy_5')) == 2
        assert len(conf.get('Dummy_6')) == 2

        # test importing main dict configuration
        # with only content of dummy_conf_dict1.yaml.
        # do not import content of dummy_conf_dict2.yaml
        conf = YmlConfLoader.get_config(
            os.path.join(
                TestYmlConfLoader.get_dummy_conf_path(),
                "dummy_conf_dict.yaml"
            ),
            ['dummy_conf_dict1.yaml']
        )
        assert Ut.is_dict(conf) and len(conf) == 5
        assert len(conf.get('Imports')) == 2
        assert len(conf.get('Dummy_1')) == 2
        assert len(conf.get('Dummy_2')) == 2
        assert len(conf.get('Dummy_3')) == 2
        assert len(conf.get('Dummy_4')) == 2
        assert conf.get('Dummy_5') is None
        assert conf.get('Dummy_6') is None

    # Test dict conf with bad import type
        with pytest.raises(YAMLFileError):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "dummy_conf_list_bad_imports.yaml"
                )
            )

    def test_get_list_main_config(self):
        """Test get_config method with list content ."""
        conf = YmlConfLoader.get_config(
            os.path.join(
                TestYmlConfLoader.get_dummy_conf_path(),
                "dummy_conf_list.yaml"
                )
        )
        assert Ut.is_list(conf) and len(conf) == 7
        assert len(conf[0].get('Imports')) == 2
        assert len(conf[1].get('Dummy_1')) == 2
        assert len(conf[2].get('Dummy_2')) == 2
        assert len(conf[3].get('Dummy_3')) == 2
        assert len(conf[4].get('Dummy_4')) == 2
        assert len(conf[5].get('Dummy_5')) == 2
        assert len(conf[6].get('Dummy_6')) == 2

        # test importing main list configuration
        # with only content of dummy_conf_dict1.yaml.
        # do not import content of dummy_conf_dict2.yaml
        conf = YmlConfLoader.get_config(
            os.path.join(
                TestYmlConfLoader.get_dummy_conf_path(),
                "dummy_conf_list.yaml"
            ),
            ['dummy_conf_list1.yaml']
        )
        assert Ut.is_list(conf) and len(conf) == 5
        assert len(conf[0].get('Imports')) == 2
        assert len(conf[1].get('Dummy_1')) == 2
        assert len(conf[2].get('Dummy_2')) == 2
        assert len(conf[3].get('Dummy_3')) == 2
        assert len(conf[4].get('Dummy_4')) == 2

        # Test list conf with bad import type
        with pytest.raises(YAMLFileError):
            YmlConfLoader.get_config(
                os.path.join(
                    TestYmlConfLoader.get_dummy_conf_path(),
                    "dummy_conf_list_bad_imports.yaml"
                )
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test App Blocks json schema validation."""
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError, ValidationError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate
from vemonitor_m8.conf_manager.loader import Loader
from .schema_test_helper import SchemaTestHelper


@pytest.fixture(name="schema_manager", scope="class")
def schema_manager_fixture():
    """Json Schema test manager fixture"""
    class SchemaManager(SchemaTestHelper):
        """Json Schema test manager fixture Class"""

        def __init__(self):
            SchemaTestHelper.__init__(self)
            self.init_data()

        def init_data(self):
            """Init data"""
            self.schema = SchemaValidate.load_schema("appBlocks")
            loader = Loader("test/conf/appBlocksTest.yaml")
            self.obj = loader.get_yaml_config()

    return SchemaManager()


class TestAppBlocksSchema:
    """Test App Blocks json schema validation."""

    def test_bad_file_key(self):
        """Test bad file key"""
        # test empty dict
        with pytest.raises(SchemaError):
            SchemaValidate.load_schema("hallo")

    def test_data_validation(self, schema_manager):
        """Test data validation"""
        assert Ut.is_list(
            SchemaValidate.validate_data(schema_manager.obj, "appBlocks"),
            not_null=True
        )

    def test_string_key_pattern(self, schema_manager):
        """Test string_key values to validate patterns"""
        datas = [
            ('name', schema_manager.obj[0]),
            ('app', schema_manager.obj[0], 'bad'),
            ('batteryBanks', schema_manager.obj[0]['args']),
            ('source', schema_manager.obj[0]['inputs']['serial'][0]),
            ('device', schema_manager.obj[0]['inputs']['serial'][0]),
            ('source', schema_manager.obj[0]['outputs']['redis'][0]),
            ('redis_node', schema_manager.obj[0]['outputs']['redis'][0]),
            ('source', schema_manager.obj[0]['outputs']['influxDb2'][0]),
            ('db', schema_manager.obj[0]['outputs']['influxDb2'][0]),
            ('measurement', schema_manager.obj[0]['outputs']['influxDb2'][0]),

            ('name', schema_manager.obj[1]),
            ('app', schema_manager.obj[1], 'bad'),
            ('batteryBanks', schema_manager.obj[1]['args']),
            ('source', schema_manager.obj[1]['inputs']['redis'][0]),
            ('redis_node', schema_manager.obj[1]['inputs']['redis'][0]),
            ('source', schema_manager.obj[1]['outputs']['influxDb2'][0]),
            ('db', schema_manager.obj[1]['outputs']['influxDb2'][0]),
            ('measurement', schema_manager.obj[1]['outputs']['influxDb2'][0]),
        ]
        schema_manager.run_test_values(datas=datas, key="string_key")

    def test_string_column_pattern(self, schema_manager):
        """Test string_column values to validate patterns"""
        datas = [
            (8, schema_manager.obj[0]['inputs']['serial'][0]['columns'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_column")

    def test_positive_number_pattern(self, schema_manager):
        """Test positive_number values to validate patterns"""
        datas = [
            ('time_interval', schema_manager.obj[0]['inputs']['serial'][0]),
            ('time_interval', schema_manager.obj[0]['outputs']['redis'][0]),
            ('time_interval', schema_manager.obj[0]['outputs']['influxDb2'][0])
        ]
        schema_manager.run_test_values(datas=datas, key="positive_number")

    def test_block_required(self, schema_manager):
        """Test block data"""

        # test required name
        val = schema_manager.obj[0]['name']
        del schema_manager.obj[0]['name']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['name'] = val

        # test required app
        val = schema_manager.obj[0]['app']
        del schema_manager.obj[0]['app']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['app'] = val

        del schema_manager.obj[0]['args']
        del schema_manager.obj[0]['inputs']
        del schema_manager.obj[0]['outputs']

        assert Ut.is_list(
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema),
            not_null=True
        )

    def test_block_args(self, schema_manager):
        """Test block data"""
        schema_manager.init_data()
        # test block args bad key
        val = schema_manager.obj[0]['args']
        schema_manager.obj[0]['args'].update(
            {'hello': schema_manager.obj[0]['args'].get('batteryBanks')}
        )
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['args'] = val

        # test empty block args
        del schema_manager.obj[0]['args']['batteryBanks']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['args'] = val

    def test_block_inputs_outputs(self, schema_manager):
        """Test block data"""
        schema_manager.init_data()
        # test block inputs bad key
        val = schema_manager.obj[0]['inputs']
        schema_manager.obj[0]['inputs'].update(
            {'badkey': schema_manager.obj[0]['inputs'].get('serial')}
        )
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['inputs'] = val

        # test empty block inputs
        del schema_manager.obj[0]['inputs']['serial']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['inputs']['serial'] = val

        # test block outputs bad key
        val = schema_manager.obj[0]['outputs']
        schema_manager.obj[0]['outputs'].update(
            {'badkey': schema_manager.obj[0]['outputs'].get('redis')}
        )
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['outputs'] = val

        # test empty block inputs
        del schema_manager.obj[0]['outputs']['redis']
        del schema_manager.obj[0]['outputs']['influxDb2']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)

    def helper_test_time_interval(self, obj, schema_manager):
        """Helper to test time interval values"""
        # test block serial time_interval
        val = obj['time_interval']
        obj['time_interval'] = 'badInt'
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['time_interval'] = val

    def helper_test_ref_cols(self, obj, schema_manager):
        """Helper to test ref cols values"""
        # test block serial empty ref_cols
        val = obj['ref_cols']
        obj['ref_cols'] = list()
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['ref_cols'] = val

        # test block serial empty ref_cols item
        val = obj['ref_cols'][0]
        obj['ref_cols'][0] = list()
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['ref_cols'][0] = val

        # test block serial ref_cols item max items
        val = obj['ref_cols'][0][0]
        obj['ref_cols'][0][0] = ['idapp', 'idcol', 'badid']
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['ref_cols'][0][0] = val

    def helper_test_columns(self, obj, schema_manager):
        """Helper test columns"""
        # test block serial empty columns
        val = obj['columns']
        obj['columns'] = list()
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['columns'] = val

        # test block serial unique columns
        val = obj['columns']
        if Ut.is_list(obj['columns'], not_null=True):
            obj['columns'].append('V')
        elif Ut.is_dict(obj['columns'], not_null=True):
            keys = list(obj['columns'].keys())
            key = keys[0]
            obj['columns'][key].append(obj['columns'][key][0])
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        obj['columns'] = val

    def test_block_serial(self, schema_manager):
        """Test block data"""
        schema_manager.init_data()
        # test block serial max items
        val = schema_manager.obj[0]['inputs']['serial']
        for i in range(0, 6):
            schema_manager.obj[0]['inputs']['serial'].append(val)
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['inputs']['serial'] = val

        # test block serial item bad key
        val = schema_manager.obj[0]['inputs']['serial'][0]
        schema_manager.obj[0]['inputs']['serial'][0].update(
            {'bad_key': "hello"})
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['inputs']['serial'][0] = val
        data = schema_manager.obj[0]['inputs']['serial'][0]
        self.helper_test_columns(data, schema_manager)
        self.helper_test_ref_cols(data, schema_manager)
        self.helper_test_time_interval(data, schema_manager)

    def test_block_redis(self, schema_manager):
        """Test block data"""
        schema_manager.init_data()
        # test block serial max items
        val = schema_manager.obj[0]['outputs']['redis']

        for i in range(0, 6):
            schema_manager.obj[0]['outputs']['redis'].append(val)

        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)

        schema_manager.obj[0]['outputs']['redis'] = val

        # test block serial item bad key
        val = schema_manager.obj[0]['outputs']['redis'][0]
        schema_manager.obj[0]['outputs']['redis'][0].update(
            {'bad_key': "hello"})
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['outputs']['redis'][0] = val

        data = schema_manager.obj[0]['outputs']['redis'][0]

        self.helper_test_columns(data, schema_manager)
        self.helper_test_ref_cols(data, schema_manager)
        self.helper_test_time_interval(data, schema_manager)

    def test_block_influx_db2(self, schema_manager):
        """Test block data influx_db2"""
        schema_manager.init_data()
        # test block serial max items
        val = schema_manager.obj[0]['outputs']['influxDb2']
        for i in range(0, 6):
            schema_manager.obj[0]['outputs']['influxDb2'].append(val)
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['outputs']['influxDb2'] = val

        # test block serial item bad key
        val = schema_manager.obj[0]['outputs']['influxDb2'][0]
        schema_manager.obj[0]['outputs']['influxDb2'][0].update(
            {'bad_key': "hello"})
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        schema_manager.obj[0]['outputs']['influxDb2'][0] = val

        data = schema_manager.obj[0]['outputs']['influxDb2'][0]

        self.helper_test_columns(data, schema_manager)
        self.helper_test_ref_cols(data, schema_manager)
        self.helper_test_time_interval(data, schema_manager)



"""Test AppBlockEvents method."""
import pytest
from vemonitor_m8.events.app_block_events import AppBlockEvents


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            self.obj = AppBlockEvents()
            self.worker_ready_event = 0
            self.worker_error_event = 0

        def init_data(self):
            """On data ready callback event."""
            self.obj = AppBlockEvents()
            self.worker_ready_event = 0
            self.worker_error_event = 0

        def on_data_ready_callback(self):
            """On data ready callback event."""
            self.worker_ready_event += 1

        def on_data_error_callback(self):
            """On data ready callback event."""
            self.worker_error_event += 1

    return HelperManager()


class TestAppBlockEvents:
    """Test AppBlockEvents method."""

    def test_worker_data_ready(self, helper_manager):
        """Test subscribe_worker_data_ready method."""
        events = helper_manager.obj
        # suscribe to event
        events.subscribe_worker_data_ready(
            obj_method=helper_manager.on_data_ready_callback
        )
        # worker_ready event counter must be equal to 0
        assert helper_manager.worker_ready_event == 0

        # trigger event on_worker_data_ready 2 times
        helper_manager.obj.worker_data_ready()
        helper_manager.obj.worker_data_ready()

        # Now worker_ready event counter must be equal to 2
        # We have triggered the event two times
        assert helper_manager.worker_ready_event == 2

        # unsuscribe to event
        events.unsubscribe_worker_data_ready(
            obj_method=helper_manager.on_data_ready_callback
        )

        # trigger event on_worker_data_ready 2 more times
        helper_manager.obj.worker_data_ready()
        helper_manager.obj.worker_data_ready()

        # worker_ready event counter must be equal to 2
        # The events triggered after unsuscribed has not effect
        assert helper_manager.worker_ready_event == 2

    def test_worker_data_error(self, helper_manager):
        """Test subscribe_worker_init_error method."""
        events = helper_manager.obj
        # suscribe to event
        events.subscribe_worker_init_error(
            obj_method=helper_manager.on_data_error_callback
        )
        # worker_error event counter must be equal to 0
        assert helper_manager.worker_error_event == 0

        # trigger event on_worker_data_ready 2 times
        helper_manager.obj.worker_init_error()
        helper_manager.obj.worker_init_error()

        # Now worker_ready event counter must be equal to 2
        # We have triggered the event two times
        assert helper_manager.worker_error_event == 2

        # unsuscribe to event
        events.unsubscribe_worker_init_error(
            obj_method=helper_manager.on_data_error_callback
        )

        # trigger event on_worker_data_ready 2 more times
        helper_manager.obj.worker_init_error()
        helper_manager.obj.worker_init_error()

        # worker_ready event counter must be equal to 2
        # The events triggered after unsuscribed has not effect
        assert helper_manager.worker_error_event == 2



"""Test redis_cache module"""
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.core.exceptions import RedisConnectionException
from vemonitor_m8.core.exceptions import RedisVeError
from vemonitor_m8.workers.redis.redis_h_time_series import HmapTimeSeriesApp


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = None
            self.host = '127.0.0.1'
            self.port = 6379
            self.db = 2
            self.node_name = "pytest"

        def init_redis_h_time_series(self):
            """Init HmapTimeSeriesApp"""
            self.obj = HmapTimeSeriesApp(
                max_rows=10,
                credentials={
                    "host": self.host,
                    "port": self.port,
                    "db": self.db
                }
            )

        def init_nodes_test(self):
            """Init HmapTimeSeriesApp"""
            # reset data cache
            self.obj.reset_node_data(
                node_name=self.node_name
            )
            # get all nodes list
            data = self.obj.get_nodes_keys_list(
                node_name=self.node_name
            )
            assert data == []
            # register nodes
            assert self.obj.register_node(
                node_name=self.node_name,
                node='pytest_1'
            ) is True
            assert self.obj.register_node(
                node_name=self.node_name,
                node='pytest_2'
            ) is True
            assert self.obj.register_node(
                node_name=self.node_name,
                node='pytest_3'
            ) is True

        def init_data_test(self):
            """Init data to test"""
            # init nodes
            self.init_nodes_test()

            assert self.obj.add_time_serie_to_node(
                time_key=1722013447,
                node='pytest_1',
                data={
                    'V': 25.5,
                    'I': 3.12
                }
            ) is True

            assert self.obj.add_time_serie_to_node(
                time_key=1722013448,
                node='pytest_1',
                data={
                    'V': 26.8,
                    'I': 1.52
                }
            ) is True

            assert self.obj.add_time_serie_to_node(
                time_key=1722013449,
                node='pytest_1',
                data={
                    'V': 26.2,
                    'I': 1.55
                }
            ) is True

        def add_more_data_test(self):
            """Init data to test"""
            now = 1722013440

            for i in range(50):
                now += 1
                assert self.obj.add_time_serie_to_node(
                    time_key=now,
                    node="pytest_1",
                    data={
                        'V': 26.8 + i + 1,
                        'I': 1.52 + i + 1
                    }
                ) is True

                if now % 2 == 0:
                    assert self.obj.add_time_serie_to_node(
                        time_key=now,
                        node="pytest_2",
                        data={
                            'V': 26.8 + i + 1,
                            'I': 1.52 + i + 1
                        }
                    ) is True

                if now % 5 == 0:
                    assert self.obj.add_time_serie_to_node(
                        time_key=now,
                        node="pytest_3",
                        data={
                            'V': 26.8 + i + 1,
                            'I': 1.52 + i + 1
                        }
                    ) is True

    return HelperManager()


class TestHmapTimeSeriesApp:
    """Test HmapTimeSeriesApp class"""

    def test_get_nodes_keys_list(self, helper_manager):
        """Test get_nodes_keys_list method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_nodes_test()

        # get all nodes list
        data = helper_manager.obj.get_nodes_keys_list(
            node_name=helper_manager.node_name
        )
        assert Ut.is_list(data, eq=3)

        # get two first nodes list
        data = helper_manager.obj.get_nodes_keys_list(
            node_name=helper_manager.node_name,
            nodes=['pytest_1', 'pytest_2']
        )
        assert Ut.is_list(data, eq=2)
        assert 'pytest_pytest_1' in data
        assert 'pytest_pytest_2' in data

        # test bad name type
        with pytest.raises(RedisVeError):
            helper_manager.obj.get_nodes_keys_list(
                node_name=["inputs_cache"]
            )

        # test loose redis conection
        helper_manager.obj.api.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_nodes_keys_list(
                node_name=helper_manager.node_name
            )

    def test_get_keys_by_node(self, helper_manager):
        """Test get_keys_by_node method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_data_test()

        # get cache keys by node
        data = helper_manager.obj.get_keys_by_node(
            formatted_node='pytest_pytest_1'
        )
        assert Ut.is_list(data, eq=3)
        assert data == [1722013447, 1722013448, 1722013449]

        # get cache keys by node
        data = helper_manager.obj.get_keys_by_node(
            formatted_node='pytest_pytest_1',
            from_time=1722013448
        )
        assert Ut.is_list(data, eq=2)
        assert data == [1722013448, 1722013449]

        # test bad name type
        with pytest.raises(RedisVeError):
            helper_manager.obj.get_nodes_keys_list(
                node_name=["inputs_cache"]
            )

        # test loose redis conection
        helper_manager.obj.api.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_nodes_keys_list(
                node_name="inputs_cache"
            )

    def test_get_interval_keys(self):
        """Test get_interval_keys method"""
        keys = [1722013465, 1722013470]
        assert HmapTimeSeriesApp.get_interval_keys(
            keys
        ) == 5

        keys = [1722013464, 1722013466, 1722013468, 1722013470]
        assert HmapTimeSeriesApp.get_interval_keys(
            keys
        ) == 2

        keys = [1722013466, 1722013467, 1722013468, 1722013469, 1722013470]
        assert HmapTimeSeriesApp.get_interval_keys(
            keys
        ) == 1

        keys = [1722013466, 1722013468, 1722013469, 1722013470, 1722013475]
        assert HmapTimeSeriesApp.get_interval_keys(
            keys
        ) == 1

    def test_get_keys_structure(self, helper_manager):
        """Test get_keys_structure method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_nodes_test()

        # Set max data cache to 10 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_more_data_test()

        result = helper_manager.obj.get_keys_structure(
            node_keys=['pytest_pytest_3', 'pytest_pytest_1', 'pytest_pytest_2'],
            from_time=1722013464,
            nb_items=2
        )
        assert result == {'pytest_pytest_3': [1722013465, 1722013470]}

        result = helper_manager.obj.get_keys_structure(
            node_keys=['pytest_pytest_3', 'pytest_pytest_1', 'pytest_pytest_2'],
            from_time=1722013460,
            nb_items=5
        )
        assert result == {
            'pytest_pytest_3': [1722013460, 1722013465, 1722013470],
            'pytest_pytest_2': [1722013472, 1722013474]
        }

    def test_enum_node_keys(self, helper_manager):
        """Test enum_node_keys method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_data_test()

        for node, keys in helper_manager.obj.enum_node_keys(
                node_name=helper_manager.node_name):

            if node == 'pytest_pytest_1':
                assert keys == [1722013447, 1722013448, 1722013449]
            else:
                assert keys is None

        for node, keys in helper_manager.obj.enum_node_keys(
                node_name=helper_manager.node_name,
                nodes=['pytest_pytest_1', 'pytest_pytest_2']):

            if node == 'pytest_pytest_1':
                assert keys == [1722013447, 1722013448, 1722013449]
            elif node == 'pytest_pytest_2':
                assert keys is None
            else:
                assert False

        for node, keys in helper_manager.obj.enum_node_keys(
                node_name=helper_manager.node_name,
                nodes=['pytest_pytest_1', 'pytest_pytest_2'],
                from_time=1722013448):

            if node == 'pytest_pytest_1':
                assert keys == [1722013448, 1722013449]
            elif node == 'pytest_pytest_2':
                assert keys is None
            else:
                assert False

    def test_reset_node_data(self, helper_manager):
        """Test reset_node_data method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_data_test()

        deleted = helper_manager.obj.reset_node_data(
            node_name=helper_manager.node_name
        )
        assert deleted == [3, 3]

    def test_add_time_serie_to_node(self, helper_manager):
        """Test add_time_serie_to_node method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_nodes_test()

        # Set max data cache to 10 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_more_data_test()

        data_pytest_pytest_1 = helper_manager.obj.api.get_hmap_data(
            "pytest_pytest_1"
        )
        data_pytest_pytest_2 = helper_manager.obj.api.get_hmap_data(
            "pytest_pytest_2"
        )
        data_pytest_pytest_3 = helper_manager.obj.api.get_hmap_data(
            "pytest_pytest_3"
        )

        assert len(data_pytest_pytest_1) == 10
        assert len(data_pytest_pytest_2) == 10
        assert len(data_pytest_pytest_3) == 10

    def test_enum_node_data_interval(self, helper_manager):
        """Test enum_node_data_interval method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_data_test()

        for key, values in helper_manager.obj.enum_node_data_interval(
            formatted_node='pytest_pytest_1',
            keys=[1722013447, 1722013448]
        ):
            if key == 1722013447:
                assert values == {'V': 25.5, 'I': 3.12}
            elif key == 1722013448:
                assert values == {'V': 26.8, 'I': 1.52}
            else:
                assert False

    def test_get_redis_time_series(self, helper_manager):
        """Test get_redis_time_series method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_nodes_test()

        # Set max data cache to 5 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_more_data_test()

        # get all items for all nodes
        result, max_time = helper_manager.obj.get_redis_time_series(
            node_name=helper_manager.node_name
        )
        keys = list(result.keys())
        keys.sort()
        assert keys == [
            1722013445, 1722013450, 1722013455,
            1722013460, 1722013465, 1722013470,
            1722013472, 1722013474, 1722013475,
            1722013476, 1722013478, 1722013480,
            1722013481, 1722013482, 1722013483,
            1722013484, 1722013485, 1722013486,
            1722013487, 1722013488, 1722013489,
            1722013490
        ] and len(keys) == 22
        assert max_time == 1722013490

        # get two items from every node who start at 1722013464
        result, max_time = helper_manager.obj.get_redis_time_series(
            node_name=helper_manager.node_name,
            from_time=1722013464,
            nb_items=2
        )
        keys = list(result.keys())
        keys.sort()
        assert keys == [
            1722013465,
            1722013470
        ]

        assert max_time == 1722013470

        # Get two items of 'V' key from pytest_1 node
        # who start at 1722013464
        result, max_time = helper_manager.obj.get_redis_time_series(
            node_name=helper_manager.node_name,
            from_time=1722013464,
            nb_items=2,
            structure={
                'pytest_1': ['V']
            }
        )
        keys = list(result.keys())
        keys.sort()
        assert keys == [1722013481, 1722013482]

        assert max_time == 1722013482

    def test_get_data_time_series(self, helper_manager):
        """Test get_data_time_series method"""
        helper_manager.init_redis_h_time_series()
        # init nodes test
        helper_manager.init_nodes_test()

        # Set max data cache to 5 items
        helper_manager.obj.set_max_rows(10)
        # add more cache data
        helper_manager.add_more_data_test()

        # get all items for all nodes
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
            node_name=helper_manager.node_name
        )
        keys = list(result.keys())
        keys.sort()
        assert keys == [
            1722013445, 1722013450, 1722013455,
            1722013460, 1722013465, 1722013470,
            1722013472, 1722013474, 1722013475,
            1722013476, 1722013478, 1722013480,
            1722013481, 1722013482, 1722013483,
            1722013484, 1722013485, 1722013486,
            1722013487, 1722013488, 1722013489,
            1722013490
        ] and len(keys) == 22
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get fist for time  keys from data cache
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
           node_name=helper_manager.node_name,
           nb_items=4
        )
        assert result == {
            1722013445: {
                'pytest_3': {'V': 31.8, 'I': 6.52}
            },
            1722013450: {
                'pytest_3': {'V': 36.8, 'I': 11.52}
            },
            1722013455: {
                'pytest_3': {'V': 41.8, 'I': 16.52}
            },
            1722013460: {
                'pytest_3': {'V': 46.8, 'I': 21.52}
            }
        }
        assert max_time == 1722013460
        assert last_time == 1722013461

        # get all items from time 1722013488
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
           node_name=helper_manager.node_name,
           from_time=1722013488
        )
        assert result == {
            1722013488: {
                'pytest_1': {'V': 74.8, 'I': 49.52},
                'pytest_2': {'V': 74.8, 'I': 49.52}
            },
            1722013489: {'pytest_1': {'V': 75.8, 'I': 50.52}},
            1722013490: {
                'pytest_1': {'V': 76.8, 'I': 51.52},
                'pytest_2': {'V': 76.8, 'I': 51.52},
                'pytest_3': {'V': 76.8, 'I': 51.52}
            }
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get all pytest_1 node data
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
           node_name=helper_manager.node_name,
           structure={'pytest_1': ['V', 'I']}
        )
        assert result == {
            1722013481: {'pytest_1': {'V': 67.8, 'I': 42.52}},
            1722013482: {'pytest_1': {'V': 68.8, 'I': 43.52}},
            1722013483: {'pytest_1': {'V': 69.8, 'I': 44.52}},
            1722013484: {'pytest_1': {'V': 70.8, 'I': 45.52}},
            1722013485: {'pytest_1': {'V': 71.8, 'I': 46.52}},
            1722013486: {'pytest_1': {'V': 72.8, 'I': 47.52}},
            1722013487: {'pytest_1': {'V': 73.8, 'I': 48.52}},
            1722013488: {'pytest_1': {'V': 74.8, 'I': 49.52}},
            1722013489: {'pytest_1': {'V': 75.8, 'I': 50.52}},
            1722013490: {'pytest_1': {'V': 76.8, 'I': 51.52}}
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get four items from every node who start at 1722013460
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
            node_name=helper_manager.node_name,
            structure={'pytest_1': ['V']}
        )

        assert result == {
            1722013481: {'pytest_1': {'V': 67.8}},
            1722013482: {'pytest_1': {'V': 68.8}},
            1722013483: {'pytest_1': {'V': 69.8}},
            1722013484: {'pytest_1': {'V': 70.8}},
            1722013485: {'pytest_1': {'V': 71.8}},
            1722013486: {'pytest_1': {'V': 72.8}},
            1722013487: {'pytest_1': {'V': 73.8}},
            1722013488: {'pytest_1': {'V': 74.8}},
            1722013489: {'pytest_1': {'V': 75.8}},
            1722013490: {'pytest_1': {'V': 76.8}}
        }
        assert max_time == 1722013490
        assert last_time == 1722013491

        # get seven items from every node who start at 1722013460
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
            node_name=helper_manager.node_name,
            from_time=1722013484,
            nb_items=4
        )

        assert result == {
            1722013484: {
                'pytest_1': {'V': 70.8, 'I': 45.52},
                'pytest_2': {'V': 70.8, 'I': 45.52}
            },
            1722013485: {
                'pytest_1': {'V': 71.8, 'I': 46.52},
                'pytest_3': {'V': 71.8, 'I': 46.52}
            },
            1722013486: {
                'pytest_1': {'V': 72.8, 'I': 47.52},
                'pytest_2': {'V': 72.8, 'I': 47.52}
            },
            1722013487: {'pytest_1': {'V': 73.8, 'I': 48.52}}
        }
        assert max_time == 1722013487
        assert last_time == 1722013488

        # Get two items of 'V' key from pytest_1 node
        # who start at 1722013464
        result, last_time, max_time = helper_manager.obj.get_data_time_series(
            node_name=helper_manager.node_name,
            from_time=1722013484,
            nb_items=4,
            structure={
                'pytest_1': ['V']
            }
        )
        assert result == {
            1722013484: {'pytest_1': {'V': 70.8}},
            1722013485: {'pytest_1': {'V': 71.8}},
            1722013486: {'pytest_1': {'V': 72.8}},
            1722013487: {'pytest_1': {'V': 73.8}}
        }
        assert max_time == 1722013487
        assert last_time == 1722013488



"""Test workers model module."""
import pytest
from vemonitor_m8.core.exceptions import SettingInvalidException
from vemonitor_m8.workers.redis.redis_app import RedisApp
from vemonitor_m8.workers.redis.redis_worker import RedisInputWorker
from vemonitor_m8.workers.redis.redis_worker import RedisOutputWorker


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = None
            self.conf = None
            self.credentials = {
                "host": '127.0.0.1',
                "port": 6379,
                "db": 2
            }

        def init_input_worker(self):
            """Init RedisInputWorker class"""
            self.conf = {
                "item": {
                    "name": "RedisInputWorkerTest",
                    "redis_node": "inputRedTst",
                    "time_interval": 1,
                    "columns": {
                        "bmv700": ["V", "I", "P"]
                    },
                    "ref_cols": [
                            ['bat_voltage', 'V'],
                            ['bat_current', 'I'],
                            ['bat_power', 'P']
                    ]
                },
                "worker_key": "RedisTestWorker",
                "enum_key": 2,
                "connector": self.credentials
            }
            self.obj = RedisInputWorker(
                conf=self.conf
            )

        def init_output_worker(self):
            """Init Workers class"""
            self.conf = {
                "item": {
                    "name": "RedisOutputWorkerTest",
                    "redis_node": "outputRedTst",
                    "time_interval": 1,
                    "cache_interval": 1,
                    "columns": {
                        "bmv700": ["V", "I", "P"]
                    },
                    "ref_cols": [
                            ['bat_voltage', 'V'],
                            ['bat_current', 'I'],
                            ['bat_power', 'P']
                    ]
                },
                "worker_key": "RedisTestWorker",
                "enum_key": 2,
                "connector": self.credentials
            }
            self.obj = RedisOutputWorker(
                conf=self.conf
            )

    return HelperManager()


class TestRedisInputWorker:
    """Test RedisInputWorker model class."""
    def test_is_ready(self, helper_manager):
        """Test is_ready method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.is_ready() is True

    def test_has_info(self, helper_manager):
        """Test has_info method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_info() is True

    def test_has_name(self, helper_manager):
        """Test is_ready method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_name() is True

    def test_has_worker_key(self, helper_manager):
        """Test has_worker_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_worker_key() is True

    def test_get_worker_key(self, helper_manager):
        """Test get_worker_key method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.get_worker_key()
        assert result == "RedisTestWorker"

    def test_has_enum_key(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_enum_key() is True

    def test_get_enum_key(self, helper_manager):
        """Test get_enum_key method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.get_enum_key()
        assert result == 2

    def test_has_time_interval(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_time_interval() is True

    def test_has_ref_cols(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_ref_cols() is True

    def test_has_worker_conf(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_worker_conf() is True

    def test_has_columns(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()

        assert helper_manager.obj.has_columns() is True

    def test_set_columns(self, helper_manager):
        """Test has_enum_key method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.set_columns(
            value={
                    "bmv700": ["V", "I", "P"]
                }
        )
        assert result is True
        result = helper_manager.obj.set_columns(
            value=["V", "I", "P"]
        )
        assert result is False

    def test_set_worker_status(self, helper_manager):
        """Test set_worker_status method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.set_worker_status()
        assert result is True

    def test_set_worker(self, helper_manager):
        """Test set_worker method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.set_worker(
            worker=helper_manager.credentials
        )
        assert result is True

        redis_app = RedisApp(
            credentials=helper_manager.credentials
        )
        result = helper_manager.obj.set_worker(
            worker=redis_app
        )
        assert result is True

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_worker(
                worker=None
            )

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_worker(
                worker={'a': 1}
            )

    def test_set_conf(self, helper_manager):
        """Test set_conf method"""
        helper_manager.init_input_worker()
        result = helper_manager.obj.set_conf(
            conf=helper_manager.conf
        )
        assert result is True

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_conf(
                conf={'a': 1}
            )


class TestRedisOutputWorker:
    """Test RedisOutputWorker model class."""
    def test_is_ready(self, helper_manager):
        """Test is_ready method"""
        helper_manager.init_output_worker()

        assert helper_manager.obj.is_ready()

    def test_has_columns(self, helper_manager):
        """Test has_columns method"""
        helper_manager.init_output_worker()

        assert helper_manager.obj.has_columns() is True

    def test_set_last_saved_time(self, helper_manager):
        """Test set_last_saved_time method"""
        helper_manager.init_output_worker()

        assert helper_manager.obj.has_last_saved_time() is False
        result = helper_manager.obj.set_last_saved_time(
            value=1722013447
        )
        assert result is True
        assert helper_manager.obj.has_last_saved_time() is True
        assert helper_manager.obj.get_last_saved_time() == 1722013447

        result = helper_manager.obj.set_last_saved_time(
            value=-1
        )
        assert result is False

    def test_set_cache_interval(self, helper_manager):
        """Test set_cache_interval method"""
        helper_manager.init_output_worker()

        assert helper_manager.obj.has_time_interval() is True
        assert helper_manager.obj.get_cache_interval() == 1
        result = helper_manager.obj.set_cache_interval(
            value=5
        )
        assert result is True
        assert helper_manager.obj.has_time_interval() is True
        assert helper_manager.obj.get_cache_interval() == 5

    def test_set_min_req_interval(self, helper_manager):
        """Test set_min_req_interval method"""
        helper_manager.init_output_worker()

        result = helper_manager.obj.set_min_req_interval(
            value=2
        )
        assert result is True
        assert helper_manager.obj.has_min_req_interval() is True
        assert helper_manager.obj.get_min_req_interval() == 2

    def test_respect_req_interval(self, helper_manager):
        """Test respect_req_interval method"""
        helper_manager.init_output_worker()

        result = helper_manager.obj.set_min_req_interval(
            value=1
        )
        assert result is True
        assert helper_manager.obj.has_min_req_interval() is True
        assert helper_manager.obj.get_min_req_interval() == 1
        helper_manager.obj.update_req_time()
        helper_manager.obj.respect_req_interval()

    def test_set_worker_status(self, helper_manager):
        """Test set_worker_status method"""
        helper_manager.init_output_worker()
        result = helper_manager.obj.set_worker_status()
        assert result is True

    def test_set_worker(self, helper_manager):
        """Test set_worker method"""
        helper_manager.init_output_worker()
        result = helper_manager.obj.set_worker(
            worker=helper_manager.credentials
        )
        assert result is True

        redis_app = RedisApp(
            credentials=helper_manager.credentials
        )
        result = helper_manager.obj.set_worker(
            worker=redis_app
        )
        assert result is True

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_worker(
                worker=None
            )

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_worker(
                worker={'a': 1}
            )

    def test_set_conf(self, helper_manager):
        """Test set_conf method"""
        helper_manager.init_output_worker()
        result = helper_manager.obj.set_conf(
            conf=helper_manager.conf
        )
        assert result is True

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_conf(
                conf={'a': 1}
            )

    def test_send_data(self, helper_manager):
        """Test send_data method"""
        helper_manager.init_output_worker()
        data = {
            1725018311: {
                'bmv700': {
                    'V': 12.065, 'I': -7.539,
                    'P': -91.0, 'CE': -65.577,
                    'SOC': 83.8, 'Alarm': 0,
                    'AR': 0, 'Relay': 0,
                    'H2': -82.854, 'H17': 68.43,
                    'H18': 85.27
                }
            },
            1725018312: {
                'bmv700': {
                    'V': 12.065, 'I': -7.524,
                    'P': -91.0, 'CE': -65.579,
                    'SOC': 83.8, 'Alarm': 0,
                    'AR': 0, 'Relay': 0,
                    'H2': -82.854, 'H17': 68.43,
                    'H18': 85.27
                }
            },
            1725018313: {
                'bmv700': {
                    'V': 12.065, 'I': -7.559,
                    'P': -91.0, 'CE': -65.581,
                    'SOC': 83.8, 'Alarm': 0,
                    'AR': 0, 'Relay': 0,
                    'H2': -82.854, 'H17': 68.43,
                    'H18': 85.27
                }
            },
            1725018314: {
                'bmv700': {
                    'V': 12.064, 'I': -7.546,
                    'P': -91.0, 'CE': -65.583,
                    'SOC': 83.8, 'Alarm': 0,
                    'AR': 0, 'Relay': 0,
                    'H2': -82.854, 'H17': 68.43,
                    'H18': 85.27
                }
            },
            1725018315: {
                'bmv700': {
                    'V': 12.064,
                    'I': -7.557, 'P': -91.0,
                    'CE': -65.585, 'SOC': 83.8,
                    'Alarm': 0, 'AR': 0,
                    'Relay': 0, 'H2': -82.854,
                    'H17': 68.43, 'H18': 85.27
                }
            }
        }
        input_structure = {
            "bmv700": [
                'V', 'I', 'P', 'CE', 'SOC',
                'Alarm', 'AR', 'Relay', 'H2', 'H17', 'H18'
            ]
        }
        result = helper_manager.obj.send_data(
            data=data,
            input_structure=input_structure
        )
        assert result is True



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test DataChecker class."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.conf_manager.data_structure_loader import DataStructureLoader
from vemonitor_m8.core.data_checker import DataChecker
from vemonitor_m8.core.exceptions import DeviceDataConfError
from vemonitor_m8.core.exceptions import DeviceInputValueError
from vemonitor_m8.core.exceptions import DeviceOutputValueError


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )
            self.obj = DataStructureLoader(
                file_names="dummy_g_conf.yaml",
                file_path=self.test_path)
            self.checkers = self.obj.get_yaml_data_structure()

    return HelperManager()


class TestDataChecker:
    """Test DataChecker class."""

    def test_validate_checker(self):
        """Test validate_checker method """
        checker = checker = {
            'name': 'Number of automatic synchronizations',
            'input_type': 'int',
            'output_type': 'float',
            'floatpoint': 0.1
        }
        assert DataChecker.validate_checker(
            key='V',
            checker=checker
        ) is True

        with pytest.raises(DeviceDataConfError):
            DataChecker.validate_checker(
                key='V',
                checker={}
            )

    def test_validate_input_value(self):
        """Test validate_input_value method """
        checker = {
            'name': 'Number of automatic synchronizations',
            'input_type': 'int',
            'output_type': 'float',
            'floatpoint': 0.1
        }
        assert DataChecker.validate_input_value(
            key='V',
            value="25525",
            checker=checker
        ) is True

        with pytest.raises(DeviceInputValueError):
            DataChecker.validate_input_value(
                key='V',
                value=25.2,
                checker=checker
            )

    def test_validate_output_value(self):
        """Test validate_output_value method """
        checker = {
            'name': 'Number of automatic synchronizations',
            'input_type': 'int',
            'output_type': 'float',
            'floatpoint': 0.1
        }
        assert DataChecker.validate_output_value(
            key='V',
            value=25.2,
            checker=checker
        ) is True

        with pytest.raises(DeviceOutputValueError):
            DataChecker.validate_output_value(
                key='V',
                value="25.2",
                checker=checker
            )

    def test_bad_checker(self, helper_manager):
        """Test check_input_columns method with bad chckers data """
        data = {
            'H10': '26',
            'H6': '-5526739',
            'H7': '11733',
            'H8': '16161',
            'H9': '368301'
        }
        local_checkers = {
            'H10': {
                'name': 'Number of automatic synchronizations',
                'floatpoint': '0.1'
            }
        }

        with pytest.raises(DeviceDataConfError):
            DataChecker.check_input_columns(
                columns=data,
                checkers=helper_manager.checkers.get('points'),
                local_checkers=local_checkers
            )

    def test_bad_floatpoint(self, helper_manager):
        """Test check_input_columns method with bad floatpoint value """
        data = {'H10': '26', 'H6': '-5526739',
                'H7': '11733', 'H8': '16161', 'H9': '368301'}

        # Test bad floatpoint type error
        local_checkers = {
            'H10': {
                'name': 'Number of automatic synchronizations',
                'input_type': 'int',
                'output_type': 'float',
                'floatpoint': '0.1'
            }
        }
        with pytest.raises(DeviceDataConfError):
            DataChecker.check_input_columns(
                columns=data,
                checkers=helper_manager.checkers.get('points'),
                local_checkers=local_checkers
            )

    def test_check_input_columns(self, helper_manager):
        """Test check_input_columns method """
        data = {'H10': '26', 'H6': '-5526739',
                'H7': '11733', 'H8': '16161', 'H9': '368301'}

        result = DataChecker.check_input_columns(
            columns=data,
            checkers=helper_manager.checkers.get('points')
        )
        assert Ut.is_dict(result, not_null=True) and len(data) == len(result)
        item = result.get('H10')
        assert Ut.is_int(item) and item == 26
        item = result.get('H6')
        assert Ut.is_float(item) and item == -5526.739
        item = result.get('H7')
        assert Ut.is_float(item) and item == 11.733
        item = result.get('H8')
        assert Ut.is_float(item) and item == 16.161
        item = result.get('H9')
        assert Ut.is_int(item) and item == 368301

    def test_check_input_columns_with_local_checkers(self, helper_manager):
        """Test check_input_columns method """
        data = {'H10': '26', 'H6': '-5526739',
                'H7': '11733', 'H8': '16161', 'H9': '368301'}

        local_checkers = {
            'H10': {
                'name': 'Number of automatic synchronizations',
                'input_type': 'int',
                'output_type': 'float',
                'floatpoint': 0.1
            }
        }
        result = DataChecker.check_input_columns(
            columns=data,
            checkers=helper_manager.checkers.get('points'),
            local_checkers=local_checkers
        )

        assert Ut.is_dict(result, not_null=True) and len(data) == len(result)
        item = result.get('H10')
        assert Ut.is_float(item) and item == 2.6

    def test_check_output_columns(self, helper_manager):
        """Test check_output_columns method """
        data = {'H10': 26, 'H6': -5526.739,
                'H7': 11.733, 'H8': 16.161, 'H9': 368301}

        result = DataChecker.check_output_columns(
            columns=data,
            checkers=helper_manager.checkers.get('points')
        )
        assert Ut.is_dict(result, not_null=True) and len(data) == len(result)
        item = result.get('H10')
        assert Ut.is_int(item) and item == 26
        item = result.get('H6')
        assert Ut.is_float(item) and item == -5526.739
        item = result.get('H7')
        assert Ut.is_float(item) and item == 11.733
        item = result.get('H8')
        assert Ut.is_float(item) and item == 16.161
        item = result.get('H9')
        assert Ut.is_int(item) and item == 368301

    def test_check_output_columns_with_local_checkers(self, helper_manager):
        """Test check_output_columns method """
        data = {'H10': 2.6, 'H6': -5526.739,
                'H7': 11.733, 'H8': 16.161, 'H9': 368301}

        local_checkers = {
            'H10': {
                'name': 'Number of automatic synchronizations',
                'input_type': 'int',
                'output_type': 'float',
                'floatpoint': 0.1
            }
        }

        result = DataChecker.check_output_columns(
            columns=data,
            checkers=helper_manager.checkers.get('points'),
            local_checkers=local_checkers
        )

        assert Ut.is_dict(result, not_null=True) and len(data) == len(result)
        item = result.get('H10')
        assert Ut.is_float(item) and item == 2.6



"""Test AsyncAppBlockRun method."""

import inspect
import time
from os import path as Opath
import pytest
from vemonitor_m8.conf_manager.config_loader import ConfigLoader
from vemonitor_m8.core.async_app_run import AsyncAppBlockRun


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )
            self.loader = ConfigLoader(self.test_path)

    return HelperManager()


class TestAsyncAppBlockRun:
    """Test AsyncAppBlockRun method."""

    def test_run_block(self, helper_manager):
        """Test subscribe_worker_data_ready method."""
        conf = helper_manager.loader.get_settings_from_schema(
            block_name=None,
            app_name="batSerialMonitor",
        )
        obj = AsyncAppBlockRun(
            conf=conf
        )

        assert obj.is_ready()

        # inputs workers blocks run on background (Threads),
        # executed by a timer
        obj.add_input_items_timer()
        # inputs workers blocks run
        obj.setup_outputs_workers()

        assert obj.workers.has_input_workers()
        assert obj.workers.has_output_workers()

        time.sleep(5)
        obj.cancel_all_timers()



# -*- coding: utf-8 -*-
"""vemonitor utils Helper"""
import re
from itertools import groupby
from typing import Union
from ve_utils.utype import UType as Ut


class Utils(Ut):
    """vemonitor utils Helper"""

    @staticmethod
    def get_min_in_loop(value: Union[int, float],
                        min_val: Union[int, float]):
        """Get the min value different from zero."""
        if value == 0:
            value = min_val
        elif 0 < min_val < value:
            value = min_val
        return value

    @staticmethod
    def get_max_in_loop(value: Union[int, float],
                        max_val: Union[int, float]):
        """Get the max value different from zero."""
        if value == 0:
            value = max_val
        elif value < max_val:
            value = max_val
        return value

    @staticmethod
    def all_equal(iterable):
        """Compare if all iterable values are equal."""
        grouped = groupby(iterable)
        return next(grouped, True) and not next(grouped, False)

    @staticmethod
    def is_valid_port(value: int):
        """Compare if all iterable values are equal."""
        return Ut.is_int(value) and 0 < value <= 65535

    @staticmethod
    def is_valid_host(value: str):
        """Compare if all iterable values are equal."""
        regex = re.compile(
            "^((?:(?:[0-1]?[0-9]?[0-9])|(?:[0-2][0-4][0-9])|(?:25[0-5]))"
            "[.](?:(?:[0-1]?[0-9]?[0-9])|(?:[0-2][0-4][0-9])|(?:25[0-5]))"
            "[.](?:(?:[0-1]?[0-9]?[0-9])|(?:[0-2][0-4][0-9])|(?:25[0-5]))"
            "[.](?:(?:[0-1]?[0-9]?[0-9])|(?:[0-2][0-4][0-9])|(?:25[0-5])))$"
        )
        return Ut.is_str(value) and Ut.is_list(regex.findall(value), eq=1)

    @staticmethod
    def is_from_time(item_time: int, from_time: int = 0):
        """Test if 0 < from_time <= item_time or from_time == 0."""
        is_valid_props = Ut.is_numeric(item_time, mini=0)\
            and Ut.is_numeric(from_time, mini=0)
        is_from_time_null = is_valid_props\
            and from_time == 0
        is_valid = is_valid_props\
            and 0 < from_time <= item_time
        return is_from_time_null \
            or is_valid



"""Test RedisApi module"""
import time
from redis.client import Pipeline
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.core.exceptions import RedisConnectionException, RedisVeError
from vemonitor_m8.workers.redis.redis_api import RedisCli
from vemonitor_m8.workers.redis.redis_api import RedisBase
from vemonitor_m8.workers.redis.redis_api import RedisApi


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = None
            self.host = '127.0.0.1'
            self.port = 6379
            self.db = 2

        def init_redis_cli(self):
            """Init RedisCli"""
            self.obj = RedisCli(
                host=self.host,
                port=self.port,
                db=self.db
            )

        def init_redis_base(self):
            """Init RedisBase"""
            self.obj = RedisBase(
                credentials={
                    "host": self.host,
                    "port": self.port,
                    "db": self.db
                }
            )

        def init_redis_api(self):
            """Init RedisApi"""
            self.obj = RedisApi(
                credentials={
                    "host": self.host,
                    "port": self.port,
                    "db": self.db
                }
            )

        def add_hmap_test(self):
            """Init RedisApi"""
            assert self.obj.flush() is True
            nb = self.obj.cli.hset(
                name='data_test',
                key='key_test_0',
                value='a'
            )
            assert nb == 1

            nb = self.obj.cli.hset(
                name='data_test',
                key='key_test_1',
                value='b'
            )
            assert nb == 1

            nb = self.obj.cli.hset(
                name='data_test',
                key='key_test_2',
                value='c'
            )
            assert nb == 1

    return HelperManager()


class TestRedisCli:
    """Test RedisCli module"""

    def test_connect_to_redis(self, helper_manager):
        """Test connect_to_redis method"""
        helper_manager.init_redis_cli()

        assert helper_manager.obj.is_ready()
        assert helper_manager.obj.is_connected()

        helper_manager.obj.connect_to_redis(
            credentials={
                "host": helper_manager.host,
                "port": helper_manager.port,
                "db": helper_manager.db
            }
        )

        assert helper_manager.obj.is_ready()
        assert helper_manager.obj.is_connected()

        with pytest.raises(RedisConnectionException):
            helper_manager.obj.connect_to_redis(
                credentials={
                    "host": "127.9.9.9",
                    "port": helper_manager.port,
                }
            )

        with pytest.raises(RedisConnectionException):
            helper_manager.obj.connect_to_redis(
                credentials={
                    "host": "127.9.9.9",
                    "port": helper_manager.port,
                    "db": "not_int",
                    "bad_prm": "bad_value"
                }
            )

    def test_set_pipeline(self, helper_manager):
        """Test set_pipeline method"""
        helper_manager.init_redis_cli()
        assert helper_manager.obj.pipe is None

        helper_manager.obj.set_pipeline()
        assert isinstance(helper_manager.obj.pipe, Pipeline)

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.set_pipeline()

    def test_flush(self, helper_manager):
        """Test _flush method"""
        helper_manager.init_redis_cli()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        data = helper_manager.obj.cli.hget(
            name='data_test',
            key='key_test_0'
        )
        assert data == 'a'

        assert helper_manager.obj.flush() is True
        data = helper_manager.obj.cli.hget(
            name='data_test',
            key='key_test'
        )
        assert data is None

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.flush()

    def test_is_redis_credentials(self, helper_manager):
        """Test is_redis_credentials method"""
        helper_manager.init_redis_cli()
        credentials = {
            "host": "127.9.9.256",
            "port": 6379
        }
        assert helper_manager.obj.is_redis_credentials(
            credentials
        ) is False

        credentials = {
            "host": "127.9.9.255",
            "port": 65536
        }
        assert helper_manager.obj.is_redis_credentials(
            credentials
        ) is False

        credentials = {
            "host": "127.9.9.255",
            "port": -1
        }
        assert helper_manager.obj.is_redis_credentials(
            credentials
        ) is False

        credentials = {
            "host": "127.9.9.9",
            "port": 6379,
            "db": "not_int",
            "bad_prm": "bad_value"
        }
        assert helper_manager.obj.is_redis_credentials(
            credentials
        ) is True

        credentials = {
            "host": "127.9.9.255",
            "port": 6379
        }
        assert helper_manager.obj.is_redis_credentials(
            credentials
        ) is True

    def test_is_redis_client_connected(self, helper_manager):
        """Test is_redis_client_connected method"""
        helper_manager.init_redis_cli()
        assert RedisCli.is_redis_client_connected(
            helper_manager.obj.cli
        ) is True

        assert RedisCli.is_redis_client_connected(
            helper_manager.obj
        ) is False


class TestRedisBase:
    """Test RedisBase module"""

    def test_get_key_type(self, helper_manager):
        """Test get_key_type method"""
        helper_manager.init_redis_base()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        data = helper_manager.obj.cli.hget(
            name='data_test',
            key='key_test_0'
        )
        assert data == 'a'

        assert helper_manager.obj.get_key_type(
            key='data_test'
        ) == 'hash'

        with pytest.raises(RedisVeError):
            helper_manager.obj.get_key_type(
                key=['data_test']
            )

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_key_type(
                key='data_test'
            )

    def test_save_redis_data_on_disk(self, helper_manager):
        """Test save_redis_data_on_disk method"""
        helper_manager.init_redis_base()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        info = helper_manager.obj.get_redis_info_usage()
        last_saved_time = info.get('rdb_last_save_time')
        now = Ut.get_int(time.time())
        assert now > last_saved_time
        assert helper_manager.obj.flush() is True
        assert helper_manager.obj.save_redis_data_on_disk() is True

        time.sleep(5)
        info = helper_manager.obj.get_redis_info_usage()
        last_saved_time = info.get('rdb_last_save_time')
        assert now <= last_saved_time

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.save_redis_data_on_disk()

    def test_get_redis_info_usage(self, helper_manager):
        """Test get_redis_info_usage method"""
        helper_manager.init_redis_base()

        info = helper_manager.obj.get_redis_info_usage(
            section='server'
        )
        assert Ut.is_dict(info, not_null=True)

        info = helper_manager.obj.get_redis_info_usage(
            keys=['redis_version', 'arch_bits']
        )
        assert Ut.is_dict(info, eq=2)

        nb_total_keys = len(helper_manager.obj.get_info_default_keys())
        info = helper_manager.obj.get_redis_info_usage()
        assert Ut.is_dict(info, eq=nb_total_keys-1)

        info = helper_manager.obj.get_redis_info_usage(
            full=True
        )
        assert Ut.is_dict(info, not_null=True)

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_redis_info_usage()


class TestRedisApi:
    """Test RedisApi module"""

    def test_get_hmap_len(self, helper_manager):
        """Test get_hmap_len method"""
        helper_manager.init_redis_api()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        nb = helper_manager.obj.get_hmap_len(
            key='data_test'
        )
        assert nb == 3

        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.get_hmap_len(
                key=['data_test']
            )

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_hmap_len(
                key='data_test'
            )

    def test_is_hmap_key(self, helper_manager):
        """Test is_hmap_key method"""
        helper_manager.init_redis_api()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        assert helper_manager.obj.is_hmap_key(
            name='data_test',
            key='key_test_0'
        ) is True

        with pytest.raises(RedisVeError):
            helper_manager.obj.is_hmap_key(
                name='data_test',
                key=['data_test']
            )

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.is_hmap_key(
                name='data_test',
                key='key_test_0'
            )

    def test_get_hmap_keys(self, helper_manager):
        """Test get_hmap_keys method"""
        helper_manager.init_redis_api()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        data = helper_manager.obj.get_hmap_keys(
            name='data_test'
        )
        assert data == ['key_test_0', 'key_test_1', 'key_test_2']

        with pytest.raises(RedisVeError):
            helper_manager.obj.get_hmap_keys(
                name=['data_test']
            )

        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_hmap_keys(
                name='data_test'
            )

    def test_del_hmap_keys(self, helper_manager):
        """Test del_hmap_keys method"""
        helper_manager.init_redis_api()

        # Add Hmap keys
        helper_manager.add_hmap_test()

        # Delete 2 hmap keys
        nb = helper_manager.obj.del_hmap_keys(
            name='data_test',
            keys=['key_test_0', 'key_test_1']
        )
        assert nb == 2

        # Control 2/3 keys are deleted
        data = helper_manager.obj.get_hmap_keys(
            name='data_test'
        )
        assert data == ['key_test_2']

        # test bad name type
        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.del_hmap_keys(
                name=['data_test'],
                keys=['key_test_0', 'key_test_1']
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.del_hmap_keys(
                name='data_test',
                keys=['key_test_0', 'key_test_1']
            )

    def test_get_hmap_data(self, helper_manager):
        """Test get_hmap_data method"""
        helper_manager.init_redis_api()
        # Add Hmap keys
        helper_manager.add_hmap_test()

        data = helper_manager.obj.get_hmap_data(
            name='data_test'
        )
        assert data == {
            'key_test_0': 'a',
            'key_test_1': 'b',
            'key_test_2': 'c'
        }

        data = helper_manager.obj.get_hmap_data(
            name='data_test',
            keys=['key_test_0', 'key_test_1']
        )

        assert data == ['a', 'b']

        data = helper_manager.obj.get_hmap_data(
            name='data_test',
            keys='key_test_0'
        )

        assert data == 'a'

        # test bad name type
        with pytest.raises(RedisVeError):
            helper_manager.obj.get_hmap_data(
                name=['data_test']
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_hmap_data(
                name='data_test'
            )

    def test_set_hmap_data(self, helper_manager):
        """Test set_hmap_data method"""
        helper_manager.init_redis_api()
        # Delete db
        assert helper_manager.obj.flush() is True

        # Test set hmap with unique str value
        nb = helper_manager.obj.set_hmap_data(
            name='data_test',
            key='key_test_0',
            values='a'
        )
        assert nb == 1
        # verrify data added
        data = helper_manager.obj.get_hmap_data(
            name='data_test'
        )
        assert data == {
            'key_test_0': 'a'
        }

        # Test set hmap with dict values
        nb = helper_manager.obj.set_hmap_data(
            name='data_test',
            values={
                'key_test_1': 'b',
                'key_test_2': 'c'
            }
        )
        assert nb == 2
        # verrify data added
        data = helper_manager.obj.get_hmap_data(
            name='data_test'
        )
        assert data == {
            'key_test_0': 'a',
            'key_test_1': 'b',
            'key_test_2': 'c'
        }

        # test bad name type
        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.set_hmap_data(
                name=['data_test'],
                key='key_test_0',
                values='a'
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.set_hmap_data(
                name='data_test',
                key='key_test_0',
                values='a'
            )

    def test_add_set_members(self, helper_manager):
        """Test add_set_members method"""
        helper_manager.init_redis_api()
        # Delete db
        assert helper_manager.obj.flush() is True

        # Test set hmap with unique str value
        nb = helper_manager.obj.add_set_members(
            name='data_test',
            values=['a']
        )
        assert nb == 1

        nb = helper_manager.obj.add_set_members(
            name='data_test',
            values=['b', 'c']
        )
        assert nb == 2

        # verrify data added
        data = helper_manager.obj.get_set_members(
            name='data_test'
        )
        assert data == {'c', 'b', 'a'}

        # test bad name type
        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.add_set_members(
                name=['data_test'],
                values=['d']
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.add_set_members(
                name='data_test',
                values=['d']
            )

    def test_remove_set_members(self, helper_manager):
        """Test remove_set_members method"""
        helper_manager.init_redis_api()
        # Delete db
        assert helper_manager.obj.flush() is True

        # Test set hmap with unique str value
        nb = helper_manager.obj.add_set_members(
            name='data_test',
            values=['a', 'b', 'c']
        )
        assert nb == 3

        # verrify data added
        data = helper_manager.obj.get_set_members(
            name='data_test'
        )
        assert data == {'c', 'b', 'a'}

        # remove data
        nb = helper_manager.obj.remove_set_members(
            name='data_test',
            values=['a', 'b']
        )
        assert nb == 2

        # verrify data removed
        data = helper_manager.obj.get_set_members(
            name='data_test'
        )
        assert data == {'c'}

        # remove data
        nb = helper_manager.obj.remove_set_members(
            name='data_test',
            values='c'
        )
        assert nb == 1

        # test bad name type
        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.remove_set_members(
                name=['data_test'],
                values=['d']
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.remove_set_members(
                name='data_test',
                values=['d']
            )

    def test_get_set_members(self, helper_manager):
        """Test get_set_members method"""
        helper_manager.init_redis_api()
        # Delete db
        assert helper_manager.obj.flush() is True

        # Test set hmap with unique str value
        nb = helper_manager.obj.add_set_members(
            name='data_test',
            values=['a', 'b', 'c']
        )
        assert nb == 3

        # verrify data added
        data = helper_manager.obj.get_set_members(
            name='data_test'
        )
        assert data == {'c', 'b', 'a'}

        # test bad name type
        with pytest.raises(RedisVeError):
            nb = helper_manager.obj.get_set_members(
                name=['data_test']
            )

        # test loose redis conection
        helper_manager.obj.cli = None
        with pytest.raises(RedisConnectionException):
            helper_manager.obj.get_set_members(
                name='data_test'
            )



"""Vemonitor main runner."""
import logging
from ve_utils.utype import UType as Ut

from vemonitor_m8.core.async_app_run import AsyncAppBlockRun
from vemonitor_m8.conf_manager.config_loader import ConfigLoader

logging.basicConfig()
logger = logging.getLogger("vemonitor")


class AppRun:
    """Run App Helper"""
    def __init__(self, block: str, app: str):
        self.conf = None
        self.app = None
        self.params = {
            "block": block,
            "app": app,
        }
        if self.load_conf():
            self.run()

    def has_params(self) -> bool:
        """Test if instance has params"""
        return Ut.is_dict(self.params)

    def load_conf(self):
        """Load configuration."""
        result = False
        if self.has_params():
            loader = ConfigLoader()
            self.conf = loader.get_settings_from_schema(
                block_name=self.params.get("block"),
                app_name=self.params.get("app"),
            )
            result = True
        return result

    def run(self) -> bool:
        """Test if instance has params"""
        if self.conf.is_valid():
            logger.info(
                "Starting app blocks workers"
            )
            self.app = AsyncAppBlockRun(self.conf)
            self.app.run_block()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test Battery Banks json schema validation."""
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError, ValidationError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate
from vemonitor_m8.conf_manager.loader import Loader
from .schema_test_helper import SchemaTestHelper


@pytest.fixture(name="schema_manager", scope="class")
def schema_manager_fixture():
    """Json Schema test manager fixture"""
    class SchemaManager(SchemaTestHelper):
        """Json Schema test manager fixture Class"""

        def __init__(self):
            SchemaTestHelper.__init__(self)
            self.init_data()

        def init_data(self):
            """Init data"""
            self.schema = SchemaValidate.load_schema("batteryBanks")
            self.obj = None

        def load_battery_bank_conf(self, key):
            """Load dummy config file"""
            if key == "batteryDatas":
                path = "test/conf/batteryBanksTest.yaml"
            elif key == "batteryBankArgs":
                path = "test/conf/batteryBankArgsTest.yaml"
            else:
                raise ValueError(
                    "Unable to load battery bank conf,"
                    f"invalid file key {key}."
                )
            self.init_data()
            loader = Loader(path)
            self.obj = loader.get_yaml_config()

    return SchemaManager()


class TestBatteryBankSchema:
    """Test Battery Banks json schema validation."""

    def test_bad_file_key(self):
        """Test bad file key"""
        with pytest.raises(SchemaError):
            SchemaValidate.load_schema("bad_key")

    def test_data_validation(self, schema_manager):
        """Test data validation"""
        schema_manager.load_battery_bank_conf('batteryDatas')
        assert Ut.is_dict(
            SchemaValidate.validate_data(schema_manager.obj, "batteryBanks"),
            not_null=True
        )

    def test_datas_string_key_pattern(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryDatas')
        datas = [
            ('battery_key',
             schema_manager.obj['batteryDatas']['bankItems']['existant']),
            ('battery_type',
             schema_manager.obj['batteryDatas']['bankItems']['existant']),
            ('battery_key',
             schema_manager.obj['batteryDatas']['bankItems']['project1']),
            ('battery_type',
             schema_manager.obj['batteryDatas']['bankItems']['project1'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_key")

    def test_datas_string_text_pattern(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryDatas')
        datas = [
            ('manufacturer',
             schema_manager.obj['batteryDatas']['batteries']['existant']),
            ('model', schema_manager.obj['batteryDatas']
             ['batteries']['existant']),
            ('manufacturer',
             schema_manager.obj['batteryDatas']['batteries']['project1']),
            ('model', schema_manager.obj['batteryDatas']
             ['batteries']['project1'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_text")

    def test_datas_positive_number(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryDatas')
        datas = [
            ('cell_voltage',
             schema_manager.obj['batteryDatas']['batteries']['existant']),
            ('bulk_voltage',
             schema_manager.obj['batteryDatas']['batteries']['existant']),
            ('float_voltage',
             schema_manager.obj['batteryDatas']['batteries']['existant']),
            ('cell_voltage',
             schema_manager.obj['batteryDatas']['batteries']['project1']),
            ('bulk_voltage',
             schema_manager.obj['batteryDatas']['batteries']['project1']),
            ('float_voltage',
             schema_manager.obj['batteryDatas']['batteries']['project1']),
        ]
        schema_manager.run_test_values(datas=datas, key="positive_number")

    def test_datas_positive_integer(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryDatas')
        datas = [
            ('in_series',
             schema_manager.obj['batteryDatas']['bankItems']['existant']),
            ('in_parallel',
             schema_manager.obj['batteryDatas']['bankItems']['existant']),
            ('nb_cells',
             schema_manager.obj['batteryDatas']['batteries']['existant']),
            ('in_series',
             schema_manager.obj['batteryDatas']['bankItems']['project1']),
            ('in_parallel',
             schema_manager.obj['batteryDatas']['bankItems']['project1']),
            ('nb_cells',
             schema_manager.obj['batteryDatas']['batteries']['project1'])
        ]
        schema_manager.run_test_values(datas=datas, key="positive_integer")

    def test_battery_bank_args_string_key_pattern(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')
        datas = [
            ('name', schema_manager.obj['batteryBankArgs']),
            ('battery_type', schema_manager.obj['batteryBankArgs']),
            ('battery_key', schema_manager.obj['batteryBankArgs'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_key")

    def test_battery_bank_args_string_text_pattern(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')
        datas = [
            ('manufacturer', schema_manager.obj['batteryBankArgs']['battery']),
            ('model', schema_manager.obj['batteryBankArgs']['battery'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_text")

    def test_battery_bank_args_positive_number(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')
        datas = [
            ('cell_voltage', schema_manager.obj['batteryBankArgs']['battery']),
            ('bulk_voltage',
             schema_manager.obj['batteryBankArgs']['battery']),
            ('float_voltage',
             schema_manager.obj['batteryBankArgs']['battery']),
        ]
        schema_manager.run_test_values(datas=datas, key="positive_number")

    def test_battery_bank_args_positive_integer(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')
        datas = [
            ('in_series', schema_manager.obj['batteryBankArgs']),
            ('in_parallel', schema_manager.obj['batteryBankArgs']),
            ('nb_cells', schema_manager.obj['batteryBankArgs']['battery'])
        ]
        schema_manager.run_test_values(datas=datas, key="positive_integer")

    def test_battery_bank_args_validation(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')

        # test batteryBankArgs invalid key
        schema_manager.obj['batteryBankArgs'].update({'badkey': 'hello'})
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)
        del schema_manager.obj['batteryBankArgs']['badkey']

        # test batteryBankArgs empty
        schema_manager.obj['batteryBankArgs'] = dict()
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)

    def test_battery_bank_args_battery(self, schema_manager):
        """Test bad key patterns on data"""
        schema_manager.load_battery_bank_conf('batteryBankArgs')

        # test batteryBankArgs empty
        schema_manager.obj['batteryBankArgs']['battery'] = dict()
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(
                schema_manager.obj, schema_manager.schema)



# -*- coding: utf-8 -*-
"""
Exceptions classes
"""


class VeMonitorError(Exception):
    """
    Settings file has not been found
    """

# ----------------
# Settings Loader Conf
# ----------------


class SettingsError(VeMonitorError):
    """
    Settings Error
    """


class SettingFileNotFound(SettingsError):
    """
    Settings file has not been found
    """


class SettingInvalidException(SettingsError):
    """
    Some data must match the expected value/type

    .. seealso:: Settings
    """


class NullSettingException(SettingsError):
    """
    Some Attributes can not be Null

    .. seealso:: Settings
    """


class SettingNotFound(SettingsError):
    """
    Some Attributes are missing

    .. seealso:: Settings
    """

# ----------------
# YAML Loader Conf
# ----------------


class LoadYamlError(VeMonitorError):
    """
    Settings Error
    """


class YAMLFileNotFound(LoadYamlError):
    """
    YAML file has not been found
    """


class YAMLFileEmpty(LoadYamlError):
    """
    YAML file empty
    """


class YAMLFileError(LoadYamlError):
    """
    YAML file empty
    """

# ----------------
# Device data check configuration
# ----------------


class DeviceDataError(VeMonitorError):
    """
    Device data check error
    """


class DeviceDataConfError(DeviceDataError):
    """
    Device data check configuration error
    """


class DeviceInputValueError(DeviceDataError):
    """
    Device Input Value Error
    """


class DeviceOutputValueError(DeviceDataError):
    """
    Device Output Value Error
    """

# ----------------
# Serial Com
# ----------------


class DataCacheError(VeMonitorError):
    """
    DataCache error
    """

# ----------------
# Serial Com
# ----------------


class SerialError(VeMonitorError):
    """
    Serial error
    """


class SerialConnectionException(SerialError):
    """
    Serial Connection error
    """


class SerialWriteException(SerialError):
    """
    VeDirect data conversion error
    """


# ----------------
# Redis
# ----------------
class RedisVeError(VeMonitorError):
    """
    Redis error
    """


class RedisConnectionException(RedisVeError):
    """
    Redis Connection error
    """


class RedisAppException(RedisVeError):
    """
    Redis App error
    """


# ----------------
# Workers
# ----------------
class WorkerException(VeMonitorError):
    """
    Worker error
    """



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Test data_structure json schema validation.
"""
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate
from vemonitor_m8.conf_manager.data_structure_loader import DataStructureLoader
from .schema_test_helper import SchemaTestHelper


@pytest.fixture(name="schema_manager", scope="class")
def schema_manager_fixture():
    """Json Schema test manager fixture"""
    class SchemaManager(SchemaTestHelper):
        """Json Schema test manager fixture Class"""

        def __init__(self):
            SchemaTestHelper.__init__(self)
            self.schema = SchemaValidate.load_schema("data_structure")
            loader = DataStructureLoader(
                "vemonitor_m8/conf_manager/confFiles/victronDeviceData.yaml"
            )
            self.obj = loader.get_yaml_data_structure(
                file_path="victronDeviceData.yaml"
            )

    return SchemaManager()


class TestSchemaDataStructure:
    """
    Test data_structure json schema validation.
    ToDo: Test all patterns in jsonshema
    """

    def test_bad_file_key(self):
        """Test bad file keys"""
        # test empty dict
        with pytest.raises(SchemaError):
            SchemaValidate.load_schema("bad_key")

    def test_data_validation(self, schema_manager):
        """Test data validation"""
        data = SchemaValidate.validate_data(
            schema_manager.obj, "data_structure")
        assert Ut.is_dict(data, not_null=True)

    def test_datas_string_key_pattern(self, schema_manager):
        """Test bad key patterns on data"""
        datas = [
            ('input_type', schema_manager.obj['points']['V']),
            ('output_type', schema_manager.obj['points']['V'])
        ]
        schema_manager.run_test_values(datas=datas, key="string_key")

    def test_string_column_pattern(self, schema_manager):
        """Test string_column values to validate patterns"""
        datas = [
            (0, schema_manager.obj['devices']['BMV']),
        ]
        schema_manager.run_test_values(datas=datas, key="string_column")

    def test_datas_positive_number(self, schema_manager):
        """Test bad key patterns on data"""
        datas = [
            ('floatpoint', schema_manager.obj['points']['V'])
        ]
        schema_manager.run_test_values(datas=datas, key="positive_number")



"""Test AppBlock jsonschema."""
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate
from vemonitor_m8.conf_manager.loader import Loader
from .schema_test_helper import SchemaTestHelper


@pytest.fixture(name="schema_manager", scope="class")
def schema_manager_fixture():
    """Json Schema test manager fixture"""
    class SchemaManager(SchemaTestHelper):
        """Json Schema test manager fixture Class"""

        def __init__(self):
            SchemaTestHelper.__init__(self)
            self.init_data()

        def init_data(self):
            """Init data"""
            self.schema = SchemaValidate.load_schema("emoncms")
            loader = Loader("test/conf/emoncmsTest.yaml")
            self.obj = loader.get_yaml_config()
            self.obj = self.obj.get("Structure")

        @staticmethod
        def get_string_description_values_helper(choice: str) -> list:
            """
            Return a list of string_auth values to test jsonschema validation.

            The function returns different lists
            depending on the choice parameter,
            which is either good or bad.
            If choice is set to good,
            then strings that are expected to be valid will be returned.
            If choice is set to bad,
            then strings that are expected not to be valid will be returned
            :param choice: Determine which list of values to return
            :return: A list of values that are bad or good string_auth.
            """
            if choice == "good":
                return [
                    "h_ello",
                    "H,eLlO",
                    "H1.eL2lO",
                    "0H1eL2lO.dev",
                    "0H1e_L2lO@.-"
                ]

            return [
                0, -1, -0.1, 0.1, 2.2,
                "_hel?lo", "_hel$lo", "_hel#lo", "_hel&lo",
                False, None, dict(), tuple()
            ]

        def get_values_helper(self, key: str, choice: str) -> list:
            """
            Return a list of values to test json schema validation.

            Accepts two parameters, key and choice.
            - 'key' is a string that specifies
                the type of value to be returned.
            - 'choice' is also a string that specifies whether
                to return good or bad values for the specified key.

            :param self: Access variables that belongs to the class
            :param key: Determine which type of values to return
            :param choice: Determine which values to return
            :return: A list of values for the given key and choice
            :doc-author: Trelent
            """
            result = None
            if Ut.is_str(key) and choice in ['good', 'bad']:
                if key == "string_key":
                    result = self.get_string_key_values_helper(choice)
                elif key == "string_column":
                    result = self.get_string_columns_values_helper(choice)
                elif key == "string_text":
                    result = self.get_string_text_values_helper(choice)
                elif key == "string_description":
                    result = SchemaManager.get_string_description_values_helper(
                        choice
                    )
                elif key == "positive_number":
                    result = self.get_positive_number_values_helper(choice)
                elif key == "positive_integer":
                    result = self.get_positive_integer_values_helper(choice)
            return result

        def run_test_values(self, datas: list, key: str):
            """Run test values helper"""
            self.run_test_values_data(
                datas=datas,
                list_values=self.get_values_helper(key, 'good'),
                choice='good'
            )

            self.run_test_values_data(
                datas=datas,
                list_values=self.get_values_helper(key, 'bad'),
                choice='bad'
            )

    return SchemaManager()


class TestEmoncmsSchemas:
    """Test AppBlock jsonschema."""
    @staticmethod
    def test_bad_file_key():
        """Test load_schema method."""
        with pytest.raises(SchemaError):
            SchemaValidate.load_schema("hallo")

    def test_data_validation(self, schema_manager):
        """Test validate_data method."""
        assert Ut.is_dict(schema_manager.obj, not_null=True)
        assert Ut.is_dict(
            SchemaValidate.validate_data(schema_manager.obj, "emoncms"),
            not_null=True
        )

    def test_string_description_pattern(self, schema_manager):
        """Test string_description values to validate patterns."""
        datas = [
            ('description', schema_manager.obj['unittest_api']['V']),
        ]
        schema_manager.run_test_values(datas=datas, key="string_description")

    def test_positive_integer_pattern(self, schema_manager):
        """Test positive_integer values to validate patterns."""
        datas = [
            ('process', schema_manager.obj['unittest_api']['V']['feeds']['V']),
            ('engine', schema_manager.obj['unittest_api']['V']['feeds']['V']),
            ('time_interval',
             schema_manager.obj['unittest_api']['V']['feeds']['V'])
        ]
        schema_manager.run_test_values(datas=datas, key="positive_integer")



"""Test config model module."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.models.config import Config
from vemonitor_m8.conf_manager.config_loader import ConfigLoader


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = Config()
            self.test_path = None
            self.loader = None

        def init_loader(self):
            """Init config loader module"""
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )
            self.loader = ConfigLoader(self.test_path)

        def init_config_with_loader(self):
            """Init config loader module"""
            self.init_loader()
            settings = self.loader.get_settings_from_schema(
                app_name="batSerialMonitor"
            )
            self.obj = settings

    return HelperManager()


class TestConfig:
    """Test Config model class."""

    def test_is_valid(self, helper_manager):
        """Test is_valid method """
        assert helper_manager.obj.is_valid() is False
        helper_manager.init_config_with_loader()
        assert helper_manager.obj.is_valid() is True

    def test___str__(self, helper_manager):
        """Test __str__ method """
        helper_manager.init_config_with_loader()
        data = str(helper_manager.obj)
        assert Ut.is_str(data, not_null=True)

    def test_serialize(self, helper_manager):
        """Test serialize method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.serialize()
        assert Ut.is_dict(data, eq=5)
